package clubs;

import infrastructure.Settings;

import java.util.ArrayList;

import uchicago.src.sim.util.Random;
import units.Household;
/**
 * Handles the working of the Funeral Club/Burial Society 
 * @author shah
 *
 */
public class FuneralClub {
	/**club id */
	private int id = -1;
	/**Initial funds to start the funeral clubs */
	private int initialFunds = Settings.FUNERALCLUB_INITIAL_MONEY;
	/**current funds of the funeral clubs */
	private int currentFunds = 0;
	/**Members of the funeral clubs */
	private ArrayList<Household> members = new ArrayList<Household>();
	/**Requests to join the funeral clubs */
	private ArrayList<Household> requests = new ArrayList<Household>();
	/**Capacity of the funeral club */
	private int capacity = 0;

	/**
	 * Default constructor
	 *
	 */
	public FuneralClub() {
	}

	/**
	 * Creates funeral club with _id
	 * @param _id
	 */
	public FuneralClub(int _id) {
		// a funeral club is open for anyone
		this.id = _id;
		this.currentFunds = this.initialFunds;
		this.capacity = Random.uniform.nextIntFromTo(11, 19);
	}

	/**
	 * Returns current funds of the funeral club
	 * @return
	 */
	public int getCurrentFunds() {
		return currentFunds;
	}
	
	/**
	 * Sets current funds of the funeral club
	 * @param currentFunds
	 */
	public void setCurrentFunds(int currentFunds) {
		this.currentFunds = currentFunds;
	}

	/**
	 * Returns id of the funeral club
	 * @return
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Sets id of the funeral club
	 * @param id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns members of the funeral club
	 * @return
	 */
	public ArrayList<Household> getMembers() {
		return members;
	}

	/**
	 * Sets members of the funeral club
	 * @param members
	 */
	public void setMembers(ArrayList<Household> members) {
		this.members = members;
	}

	/**
	 * Add household as member
	 * @param _member
	 */
	public void addMember(Household _member) {
		this.members.add(_member);
	}

	/**
	 * Remove household from the funeral club
	 * @param _member
	 */
	public void removeMember(Household _member) {
		this.members.remove(_member);
	}

	/**
	 * add contribution to the fund
	 * @param money
	 */
	public void addContribution(int money) {
		this.currentFunds += money;
	}

	/**
	 * If membership less than capacity
	 * @return
	 */
	public boolean handleRequestMembership() {
		return true ? this.members.size() < this.capacity : false;
	}

	/**
	 * Funeral clubs/burial societies give full amount to
	 * some, and a bit proportion of the rest pool to the others who've suffered
	 * a breavement. This is a stylized implementation. 
	 */
	public void step() {
		// first determine how many requests we have
		int numRequests = this.requests.size();
		// can we give the funeral cost to all of them?
		int share = 0;
		if (this.currentFunds - (Settings.FUNERAL_COST * numRequests) >= 0) {
			share = Settings.FUNERAL_COST;
		}
		// else we give the equal share of money to people
		else {
			share = this.currentFunds / numRequests;
		}

		for (Household household : this.requests) {
			// pay(household, share);
			// household.addWealth(share);
			household.addFuneralWealth(share);
		}
		this.requests.clear();
	}

	/**
	 * Household pays its share
	 * @param household
	 * @param share
	 */
	public void pay(Household household, int share) {
		household.addWealth(share);
	}

	/**
	 * If a households defaults for some time and doesn't pay, then it doesn't get any
	 * help from the club, anymore after a grace period.
	 */
	public void request(Household household) {
		if ((household.getDefaultFuneralClubCount() <= Settings.FUNERALCLUB_GRACE)
				&& (!this.requests.contains(household))) {
			this.requests.add(household);
		} else {
			this.members.remove(household);
		}
	}
}