package clubs;

import infrastructure.Settings;

import java.util.ArrayList;

import display.Model;
import uchicago.src.sim.util.SimUtilities;
import units.Agent;

/**
 * TODO: need mechansim for handling requests for getting money out of turn ...
 * if more than one agents request, we need to maintain a priority queuue so
 * that which is 'supposed' to be just ... if there is a delay due to the size
 * of members and/or number of requests, a member may opt to leave the saving
 * club and join another but it's a kind of tension ... because a savings club
 * which is older and larger may attract more people than otherwise ... if
 * there're many members, then you can get a much larger amount of money ... but
 * on the cost that your turn comes late too as the consquence
 * 
 * people get togeter to create a savings scheme ... "it appears that some
 * people might be involved in more than one scheme while others are involved in
 * no schemes."
 * 
 */

public class SavingsClub {

	/** a savings club may either be disbanded or a new cycle starts */
	private int cycle = 0, months = 0;
	private Agent moderator = null;
	private ArrayList<Agent> candidates = new ArrayList<Agent>(), 
							members = new ArrayList<Agent>();
	private Model model;
	private boolean viable = true;

	/**
	 * Default constructor
	 */
	public SavingsClub() {
	}

	/**
	 * Creates a stokvel (savings club) with a moderator
	 * @param _moderator
	 * @param _members
	 * @param _model
	 */
	public SavingsClub(Agent _moderator, ArrayList<Agent> _members, Model _model) {		
		this.members = new ArrayList<Agent>(_members);
		this.moderator = _moderator;
		this.model = _model;
		candidates = new ArrayList<Agent>(_members);
		initialize();
	}

	/** Reinitializing the stokvel after every cycle */
	public void initialize() {
		SimUtilities.shuffle(candidates);
		this.cycle++;
	}
	
	/** Choose a random benefactor */
	public Agent chooseRandomNextBenefactor() {
		if (!members.isEmpty()) {
			SimUtilities.shuffle(members);
			Agent agent = (Agent) members.remove(0);
			return agent;
		} else
			return null;
	}

	/** Called from the model. Handles members' pulling out the
	 * stokvels and also if the stokvel needs to be reinitialized. */
	public void step() {
		ArrayList<Agent> temp = new ArrayList<Agent>(this.members);
		for (Agent agent : temp) {
			boolean flag = false;
			if (!model.getAgentList().contains(agent)) {
				if (this.moderator == agent)
					this.moderator = null;
				flag = true;
			}
			if (agent.isPullOutMembership()) {
				flag = true;
			}
			if (flag) {
				this.members.remove(agent);
				agent.setMemberSavings(false);
				// might already be removed from this list
				this.candidates.remove(agent);
			}
		}
		// if we're still viable and the minimum members hold ...
		if (this.members.size() >= Settings.MIN_SAVINGS_MEMBERS) {
			if (this.moderator == null) {
				moderator = (Agent) this.members.get(0);
				moderator.setInnovator(true);
			}
			if (!this.candidates.isEmpty()) {
				mainStep();
			} else {
				/* what's missing here is that there's no chance for people to
				 * join in the middle ... so currently we can't use the restart function ...
				 * instead, we demolish the club after a single rota
				 */
				restart();
			}
		} else {
			// we're no more viable ... we demolish the club ...
			abandon();
		}
	}

	/**
	 * for restarting the club, we have got to check if we've the remaining
	 * candidates intact and if they're viable for the club. We might also
	 * like to invite more people who're not part of the cycle before
	 */

	public void restart() {
		cycle++;
		this.candidates = new ArrayList<Agent>(this.members);
	}

	/**
	 * Main step for stokves. Gets the recipient for the current terms.
	 *
	 */
	public void mainStep() {
		this.months++;
		int totalPool = 0;
		for (int i = 0; i < this.members.size(); i++) {
			totalPool += Settings.STOKVEL_AMOUNT;
		}
		SimUtilities.shuffle(this.candidates);
		Agent recipient = (Agent) this.candidates.get(0);
		recipient.setSavingsPool(totalPool);
		this.candidates.remove(recipient);
	}

	/**
	 * Abandon the stokvel when it is no longer viable.	 *
	 */
	public void abandon() {
		this.viable = false;
		for (Agent agent : (ArrayList<Agent>) this.members) {
			agent.flushMembership();
		}
	}

	public static void main(String[] args) {
	}

	/**
	 * Returns the number of cycles for this stokvel.
	 * @return
	 */
	public int getCycle() {
		return cycle;
	}

	/**
	 * Returns the moderator for this stokvel.
	 * @return
	 */
	public Agent getModerator() {
		return moderator;
	}
	
	/**
	 * Sets the moderator for this stokvel.
	 * @param moderator
	 */
	public void setModerator(Agent moderator) {
		this.moderator = moderator;
	}

	/**
	 * Sets the cycle for this stokvels.
	 * @param cycle
	 */
	public void setCycle(int cycle) {
		this.cycle = cycle;
	}

	/**
	 * Returns the number of months this stokvel is running.
	 * @return
	 */
	public int getMonths() {
		return months;
	}

	/**
	 * If this stokvel is viable or not. 
	 * @return
	 */
	public boolean isViable() {
		return viable;
	}

	/**
	 * Sets the viability status of the stokvel. 
	 */
	public void setViable(boolean viable) {
		this.viable = viable;
	}
	
	/**
	 * Returns the member list of the stokvels
	 * @return
	 */
	public ArrayList<Agent> getMembers() {
		return members;
	}
	
	/**
	 * Sets the member list of the stokvels
	 * @param members
	 */
	public void setMembers(ArrayList<Agent> members) {
		this.members = members;
	}
}