package display;

import infrastructure.Settings;

import java.util.ArrayList;

import uchicago.src.sim.util.Random;

import units.Agent;

/**
 * AgentNode type which is a subclass of the Node
 *
 */
public class AgentNode extends Node {

	/** Here single individual (agent) is the node */
	private Agent agent;

	/**
	 * Default constructor
	 *
	 */
	public AgentNode() {
		super();
	}
	
	/**
	 * Sets agent to its node 
	 * @param _agent
	 */
	public AgentNode(Agent _agent) {
		setAgent(_agent);
	}

	/**
	 * Retutrns the agent for this node/
	 */
	public Agent getAgent() {
		return agent;
	}

	/**
	 * Sets the agent for this node.
	 * @param agent
	 */
	public void setAgent(Agent agent) {
		this.agent = agent;
	}

	/** This is needed for tests in contains() operation of node lists */	
	public boolean equals(Object o) {
		if (this == o)
			return true; // identity
		if ((o == null) || !(o instanceof AgentNode))
			return false; // null reference or not an agent node
		AgentNode otherNode = (AgentNode) o;
		if (this.agent != null && this.agent == otherNode.getAgent())
			return true; // refers to same agent
		return false;
	}
	
	/**
	 * Adaptation from the Repast's implementation for JinGirNew Model.
	 * @param list
	 */
	public void meetRandom(ArrayList list) {
		int index = Random.uniform.nextIntFromTo(0, list.size() - 1);
		AgentNode node = (AgentNode)list.get(index);
		while (this.equals(node)) {
			index = Random.uniform.nextIntFromTo(0, list.size() - 1);
			node = (AgentNode)list.get(index);
		}
		if (checkFriendshipLimit(node)) {
			makeEdgeToFrom(node);
			Agent me = (Agent) this.agent;
			Agent friend = (Agent) node.getAgent();
			if (!me.getFriends().contains(friend)) {
				me.getFriends().add(friend);	
			}
			if (!friend.getFriends().contains(me)) {
				friend.getFriends().add(me);
			}			
		}		
	}
	
	/**Adaptation from the Repast's implementation for JinGirNew Model. */
	public void meetNeighbor() {
		if (getOutDegree() > 1) {
			AgentNode jNode = (AgentNode)getRandomNodeOut();
			AgentNode kNode = (AgentNode)getRandomNodeOut();

			while (jNode.equals(kNode)) {
				kNode = (AgentNode)getRandomNodeOut();
			}
			if (jNode.checkFriendshipLimit(kNode)) {
				jNode.makeEdgeToFrom(kNode);
				Agent jAgent = (Agent) jNode.getAgent();
				Agent kAgent = (Agent) kNode.getAgent();
				if (!jAgent.getFriends().contains(kAgent)) {
					jAgent.getFriends().add(kAgent);	
				}
				if (!kAgent.getFriends().contains(jAgent)) {
					kAgent.getFriends().add(jAgent);
				}			
			}	
		}
	}
	
	/** Adaptation from the Repast's implementation for JinGirNew Model. */
	public void removeFriend() {
		AgentNode jNode = (AgentNode)getRandomNodeOut();
		// will be null if no outEdges.
		if (jNode != null) {
			removeEdgesTo(jNode);
			jNode.removeEdgesFrom(this);
			removeEdgesFrom(jNode);
			jNode.removeEdgesTo(this);
			Agent me = (Agent) this.agent;
			Agent jAgent = (Agent) jNode.getAgent();
			if (me.getFriends().contains(jAgent)) {
				me.getFriends().remove(jAgent);
			}
			if (jAgent.getFriends().contains(me)) {
				jAgent.getFriends().remove(me);
			}
		}
	}
	
	/** 
	 * Check if this node can have an additional friend given the friendship upperlimit.
	 * @param node
	 * @return
	 */
	public boolean checkFriendshipLimit(AgentNode node) {
		if ((! hasEdgeTo(node)) && getOutDegree() < Settings.FRIENDSHIP_UPPER_LIMIT &&
				node.getOutDegree() < Settings.FRIENDSHIP_UPPER_LIMIT) {
			return true;
		}
		return false;
	}
}