package display;

import java.io.*;
import java.util.*;

/**
 * Class to record affiliation/2-mode network for Pajek/UCI
 * 
 */
public class MultiAffiliation {		
	private ArrayList<Node> actors = new ArrayList<Node>();
	private ArrayList <ArrayList> events = new ArrayList<ArrayList>();
	
	HashMap<Node, ArrayList<Integer>> affiliation = new HashMap<Node, ArrayList <Integer>>();
	
	public void populateAffiliation() {		
		for (Node actor : actors) {		
			String label = actor.getNodeLabel();
			int counter = 1;
			ArrayList<Integer> memberships = new ArrayList<Integer>();
			for (ArrayList<Node> nodeList : events) {				
				for (Node node : nodeList) {
					String label2 = node.getNodeLabel();					
					if (label.compareTo(label2) == 0) {
						memberships.add(new Integer(counter));
					}
				}
				counter++;
			}
			affiliation.put(actor, memberships);
		}
	}
	
	/**
	 * Adapted and extended from Friedrich/Richard
	 * @param prefix
	 */
	public void saveAffiliationNetworkPajek(String prefix) {
		PrintWriter netRecorder;

		try {
       	 netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
    				"" + prefix + ".net")));		
       	 
       	 int size = actors.size() + events.size();
    	 
       	 netRecorder.println("*Vertices" + " " + size + " " + actors.size());
       	 
    	 //first recording actors
       	 for (Node actor : actors) {
       		 netRecorder.println(actors.indexOf(actor) + 1  + " \"Actor "
    				 + actor.getNodeLabel() + "\"");   		 
       	 }
       	 
       	 int numActors = actors.size();
       	 
    	 //second, recording events
       	 for (int i=1; i<=events.size(); i++) {
    		 netRecorder.println(i + numActors + " \" Event " + i + "\"");   			        		 
       	 }
       	 
       	 populateAffiliation();
       	     	 
    	 // now the edges
    	 netRecorder.println("*Edges");
    	 
    	 for (Node actor : actors) {
    		 ArrayList<Integer> memberships = (ArrayList<Integer>) affiliation.get(actor);
    		 for (Integer num : memberships) {
                 netRecorder.println((actors.indexOf(actor) + 1) + " "
                		 + (num + actors.size()));   			 
    		 }
    	 }    	
    	 netRecorder.close();       	     	     	     	 
		}catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @param prefix
	 */
	public void saveAffiliationNetworkUCI(String prefix) {
		PrintWriter netRecorder;

		try {
       	 netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
    				"" + prefix + ".dat")));
       	 
       	 netRecorder.println("dl " + "nr=" + actors.size() + " "  + "nc="+ events.size()
       			 + " " + "format = nodelist2ROW");
       	 
       	netRecorder.println("row labels embedded");
       	netRecorder.println("col labels embedded");
       	netRecorder.println("data:");

		populateAffiliation();
		
    	 for (Node actor : actors) {
    		 String eventsLabel = "";
    		 ArrayList<Integer> memberships = (ArrayList<Integer>) affiliation.get(actor);
    		 for (Integer num : memberships) {
    			 eventsLabel += "Event:" + + (num) + "       ";                   			     			 
    		 }   		 
    		 netRecorder.println((actors.indexOf(actor) + 1) 
            		 + "       " + eventsLabel);
    	 }    	
    	 netRecorder.close();       	     	     	     	 
		}catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public ArrayList<ArrayList> getEvents() {
		return events;
	}

	public void setEvents(ArrayList<ArrayList> actorsCollection) {
		this.events = actorsCollection;
	}

	public ArrayList<Node> getActors() {
		return actors;
	}
}