package display;

import java.awt.Color;
import java.util.ArrayList;
import uchicago.src.sim.gui.RectNetworkItem;
import uchicago.src.sim.network.DefaultDrawableNode;
import uchicago.src.sim.network.DefaultNode;

/**
 * Node class. SandboxNode extends DefaultDrawableNode and so it is both a
 * DefaultNode and a DrawableNonGridNode. All of the important behavoir here as
 * to do with creating new links.
 * 
 * -
 * This is based and with new functions on the code by Friedrich for multilayer network
 * model ...
 */

public abstract class Node extends DefaultDrawableNode {
	int maxDegree = -1;
	double initEdgeStrength = -1;
	
	private int ID = -1;

	public Node() {
	}

	/**
	 * Initialize this saNode, this takes the place of the constructor when a
	 * SandboxNode is created from a file using its no-arg constructor.
	 */
	public void init(int x, int y) {
		RectNetworkItem rect = new RectNetworkItem(x, y);
		setDrawable(rect);
	}

	/**
	 * Clears all the links for this node
	 */
	public void clearEdges() {
		ArrayList<Node> fromNodes = (ArrayList<Node>) this.getFromNodes();
		ArrayList<Node> toNodes = (ArrayList<Node>) this.getToNodes();
		for (Node node : fromNodes) {
			node.removeEdgesFrom(this);
		}
		for (Node node : toNodes) {
			node.removeEdgesTo(this);
		}
		clearInEdges();
		clearOutEdges();
	}

	// undirected edges
	public void constructEdges(ArrayList<Node> nodes) {
		for (Node node : nodes) {
			if (node != null) {
				makeEdgeToFrom(node);	
			}			
		}
	}

	// undirected edges
	public void constructEdges(ArrayList<Node> nodes, Color color) {
		for (Node node : nodes) {
			makeEdgeToFrom(node, color);
		}
	}

	/**
	 * 
	 * Remove a link randomly from this node
	 */
	public void removeRandomLink() {
		Node randomNode = (Node) this.getRandomNodeOut();
		if (randomNode != null) {
			this.removeEdgesTo(randomNode);
			randomNode.removeEdgesFrom(this);
			this.removeEdgesFrom(randomNode);
			randomNode.removeEdgesTo(this);
		}
	}

	/**
	 * Makes an edge to the specified node and from the specifed node to this
	 * saNode if both nodes do not already have edges to each other and if
	 * adding the edge keeps their degrees less than maxDegree. If an edge
	 * allready exitsts, its strength is refreshed.
	 * 
	 * @param node
	 * @param maxDegree
	 */
	public void makeEdgeToFrom(DefaultNode node) {
		if (hasEdgeTo(node)) {
			ArrayList<SAEdge> outE = new ArrayList<SAEdge>(getOutEdges());
			for (SAEdge e : outE) {
				if (node.equals(e.getTo())) {
//					e.resetStrength();
				}
			}
			outE = new ArrayList<SAEdge>(node.getOutEdges());
			for (SAEdge e : outE) {
				if (this.equals(e.getTo())) {
//					e.resetStrength();
				}
			}
		} else {
			// if (getOutDegree() < maxDegree && node.getOutDegree() <
			// maxDegree) {
//			 SAEdge edge = new SAEdge(this, node, Color.RED);
			SAEdge edge = new SAEdge(this, node);
			addOutEdge(edge);
			node.addInEdge(edge);
//			 SAEdge otherEdge = new SAEdge(node, this, Color.RED);
			SAEdge otherEdge = new SAEdge(node, this);
			node.addOutEdge(otherEdge);
			addInEdge(otherEdge);
		}
	}

	/**
	 * Makes an edge to the specified node and from the specifed node to this
	 * saNode if both nodes do not already have edges to each other and if
	 * adding the edge keeps their degrees less than maxDegree. If an edge
	 * allready exitsts, its strength is refreshed.
	 * 
	 * @param node
	 * @param maxDegree
	 */
	public void makeEdgeToFrom(DefaultNode node, Color color) {
		if (hasEdgeTo(node)) {
			ArrayList<SAEdge> outE = new ArrayList<SAEdge>(getOutEdges());
			for (SAEdge e : outE) {
				if (node.equals(e.getTo())) {
//					e.resetStrength();
				}
			}
			outE = new ArrayList<SAEdge>(node.getOutEdges());
			for (SAEdge e : outE) {
				if (this.equals(e.getTo())) {
//					e.resetStrength();
				}
			}
		} else {
			// if (getOutDegree() < maxDegree && node.getOutDegree() <
			// maxDegree) {
			SAEdge edge = new SAEdge(this, node, color);
			addOutEdge(edge);
			node.addInEdge(edge);
			SAEdge otherEdge = new SAEdge(node, this, color);
			node.addOutEdge(otherEdge);
			addInEdge(otherEdge);
		}
	}


	/**
	 * @return Returns the maxDegree.
	 */
	public int getMaxDegree() {
		return maxDegree;
	}

	/**
	 * @param maxDegree
	 *            The maxDegree to set.
	 */
	public void setMaxDegree(int maxDegree) {
		this.maxDegree = maxDegree;
	}

	// force subclasses to implement equals() method
	abstract public boolean equals(Object o);

	public void removeNode(DefaultNode node) {
		this.removeEdgesTo(node);
		node.removeEdgesFrom(this);
		this.removeEdgesFrom(node);
		node.removeEdgesTo(this);
	}

	public int getID() {
		return ID;
	}

	public void setID(int id) {
		this.ID = id;
	}
}