package economics;

import infrastructure.Settings;

import java.util.ArrayList;

import display.Model;
import units.Agent;

/*******************************************************************************
 * this is a a special kind of agent / or agency, so-called 'Finance Agency' ...
 * created as an instituition responsible for allocating grants to agents .. *
 */

public class FinanceAgency {
	private Model model;

	public FinanceAgency(Model _model) {
		this.model = _model;
	}

	/**
	 * allocate or stop grants to eligible agents
	 *
	 */
	public void step() {
		ArrayList<Agent> agents = new ArrayList<Agent>();
		agents = (ArrayList<Agent>) model.getAgentList();
		for (Agent agent : agents) {
			allocateGrant(agent);
		}
	}

	/**
	 * Allocate grant to agent is eligible
	 * @param agent
	 */
	private void allocateGrant(Agent agent) {
		if (agent.getGrantValue() == Settings.GRANTS.NO_GRANT) {
			checkEligibility(agent);
		}
		else if (agent.getGrantValue() == Settings.GRANTS.DISABILITY_GRANT) {
			allocatePension(agent);
		}
		else if (agent.getGrantValue() == Settings.GRANTS.CHILD_GRANT
				||  agent.getGrantValue()== Settings.GRANTS.FOSTER_CARE_GRANT){			
			checkStopGrant(agent);
		}		
	}

	/**
	 * Check if the grant needs to be stopped for this agent
	 * @param agent
	 */
	private void checkStopGrant(Agent agent) {
		if (agent.getAge() >= Settings.GRANTS.CHILD_GRANT_AGE 
				&& agent.getGrantValue() == Settings.GRANTS.CHILD_GRANT) {
			agent.setGrant(Settings.GRANTS.NO_GRANT);			
		}
		else if (agent.getAge() >= Settings.GRANTS.FOSTER_CARE_GRANT_AGE
				&& agent.getGrantValue() == Settings.GRANTS.FOSTER_CARE_GRANT) {
			agent.setGrant(Settings.GRANTS.NO_GRANT);
		}		

	}
	
	/**
	 * Check eligbility for the agent
	 * @param agent
	 */
	private void checkEligibility(Agent agent) {
		if (agent.getAge() <= Settings.GRANTS.FOSTER_CARE_GRANT_AGE 
				&& agent.isOrphan()				
				&& agent.isWouldGetChildGrant()
		) {
			agent.setGrant(Settings.GRANTS.FOSTER_CARE_GRANT);
			return;
		}
		int ageGroup = agent.getAgeGroup();
		switch(ageGroup) {
		case Settings.AGE_GROUP.CHILD:
			if (agent.getAge() <= Settings.GRANTS.CHILD_GRANT
					&& !agent.isOrphan()					
					&& agent.isWouldGetChildGrant()
			) {
				agent.setGrant(Settings.GRANTS.CHILD_GRANT);
			}		
			break;
		case Settings.AGE_GROUP.ADULT:
			if ((!agent.isLaborFit((int) model.getTickCount())					
					&& Math.random() < Settings.GRANTS.DISABILITY_GRANT_PROBABILITY
					&& agent.isWouldGetDisabilityGrant()
			)
			|| agent.getGrantValue() == Settings.GRANTS.DISABILITY_GRANT
			){
				agent.setGrant(Settings.GRANTS.DISABILITY_GRANT);				
			}						
			break;
		case Settings.AGE_GROUP.SENIOR:
			allocatePension(agent);
			break;
		default: break;
		}
	}

	/**
	 * allocate pension to this agent
	 * @param agent
	 */
	private void allocatePension(Agent agent) {
		if (((agent.getGender() == Settings.GENDER.FEMALE 
				&& agent.getAge() >= Settings.GRANTS.PENSION_AGE_FEMALE) 
				|| (agent.getGender() == Settings.GENDER.MALE 
						&& agent.getAge() >= Settings.GRANTS.PENSION_AGE_MALE))
						&& agent.isWouldGetPension()
		) {
			agent.setGrant(Settings.GRANTS.PENSION_GRANT);
		}		
	}
}