/** this class using * 
 * METHODS FOR MODELING THE HIV/AIDS EPIDEMIC
IN SUB-SAHARAN AFRICA
Joshua A. Salomon
Emmanuela E. Gakidou
Christopher J.L. Murray
GPE Discussion Paper Series: No. 3
EIP/GPE/EBD
 */
package health;

import infrastructure.Settings;
import units.Agent;
import cern.jet.random.Gamma;
import cern.jet.random.engine.MersenneTwister;

/**
 * TO DO: Plausible modelling of prevalence of HIV/AIDS
 * in Limpopo the prevalence rate for men is 9.5%, for women it's 13.5% 
 * also 1/40% of total population are children
 * 
 */

public class AidsSpread {	
	Gamma gamma = new Gamma(10, 2.5, new  MersenneTwister());
	double count = 0;
	public static final class GAMMA {
		public static final double ALPHA = 0.19;
		public static final double BETA = 0.55; //
		public static final double THETA = 0.35;
	}

	public AidsSpread() {		
	}	

	/**
	 * Returns the incidence based on the Gamma function for COLT library
	 * @return
	 */
	public double returnIncidence() {	
		double gamma = Gamma.staticNextDouble(GAMMA.ALPHA, GAMMA.BETA);
		count += gamma;
		System.out.println("   Incidence: " + Math.ceil(gamma));		
		return gamma;
	}	

	/**
	 * Returns the gamma value for own implementation
	 * @param tick
	 * @return
	 */
	public double returnGamma(double tick) {
		double numerator = Math.pow(GAMMA.BETA, -GAMMA.ALPHA) * Math.pow(tick, GAMMA.ALPHA-1) * Math.exp(-tick/GAMMA.BETA);
		double denominator = cern.jet.stat.Gamma.gamma(GAMMA.ALPHA);
		return Math.ceil(10E2 * numerator/denominator);
	}

	public double returnExtendedGamma(double tick) {
		double gamma = (double) returnGamma(tick);
		if (tick <= GAMMA.BETA * (GAMMA.ALPHA-1)) {
			return Math.ceil(10E2*gamma);
		}
		else {
			double firstComponent = gamma * (1-GAMMA.THETA);
			double secondComponent = returnGamma(GAMMA.BETA*GAMMA.ALPHA-1);
			secondComponent *= GAMMA.THETA;			
			return Math.ceil(10E2*(firstComponent + secondComponent));				
//			return Math.ceil(returnIncidence());
		}		
	}
	/**AVERT.ORG
	 * Age (years)  	Male prevalence %  	Female prevalence %
		2-4 					4.9 			5.3
		5-9 					4.2			 	4.8
		10-14				 	1.6			 	1.8
		15-19				 	3.2			 	9.4
		20-24				 	6.0			 	23.9
		25-29				 	12.1		 	33.3
		30-34				 	23.3		 	26.0
		35-39				 	23.3		 	19.3
		40-44				 	17.5		 	12.4
		45-49 					10.3		 	8.7
		50-54				 	14.2		 	7.5
		55-59				 	6.4			 	3.0
		60+					 	4.0			 	3.7
		Total				 	8.2			 	13.3	 * 
	 */
	public boolean returnAgentPrevalence(Agent agent) {
		double chance = Math.random();
		int gender = agent.getGender();
		boolean isHIV = false;
		int age = agent.getAge();
		if (age >= 2 && age <= 4) {
			isHIV = chance < 0.05 ? true : false;
		}
		else if (age >= 5 && age <= 9) {
			isHIV = chance < 0.05 ? true : false;
		}	
		else if (age >= 10 && age <= 14) {
			isHIV = chance < 0.02 ? true : false;
		}
		else if (age >= 15 && age <= 19) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.094 ? true : false;	
			}
			else {
				isHIV = chance < 0.03 ? true : false;
			}						
		}
		else if (age >= 20 && age <= 24) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.23 ? true : false;	
			}
			else {
				isHIV = chance < 0.06 ? true : false;
			}						
		}
		else if (age >= 25 && age <= 29) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.333 ? true : false;	
			}
			else {
				isHIV = chance < 0.121 ? true : false;
			}						
		}
		else if (age >= 30 && age <= 34) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.26 ? true : false;	
			}
			else {
				isHIV = chance < 0.233 ? true : false;
			}						
		}
		else if (age >= 35 && age <= 39) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.193 ? true : false;	
			}
			else {
				isHIV = chance < 0.121 ? true : false;
			}						
		}
		else if (age >= 40 && age <= 44) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.124 ? true : false;	
			}
			else {
				isHIV = chance < 0.174 ? true : false;
			}						
		}
		else if (age >= 45 && age <= 49) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.075 ? true : false;	
			}
			else {
				isHIV = chance < 0.142 ? true : false;
			}						
		}
		else if (age >= 50 && age <= 54) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.333 ? true : false;	
			}
			else {
				isHIV = chance < 0.121 ? true : false;
			}						
		}
		else if (age >= 55 && age <= 59) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.03 ? true : false;	
			}
			else {
				isHIV = chance < 0.064 ? true : false;
			}						
		}
		else if (age >= 60) {
			if (gender == Settings.GENDER.FEMALE) {
				isHIV = chance < 0.037 ? true : false;	
			}
			else {
				isHIV = chance < 0.04 ? true : false;
			}						
		}
		return isHIV;
	}
	
/*	
	public WHOLifeTable malesTable = new WHOLifeTable(
			new int[]{0, 1, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 
					70, 75, 80, 85, 90, 95, 100},
					new double[]{
					0.05769,//0-1
					0.01474,//1-5
					0.01903,//
					0.00694,
					0.01451,
					0.037,
					0.05493,
					0.11001,
					0.1725,
					0.14059,
					0.15916,
					0.167,
					0.16182,
					0.19377,
					0.23759,
					0.31431,
					0.4283,
					0.57377,
					0.73448,
					0.82419,
					0.87064,
					1});

	public WHOLifeTable femalesTable = new WHOLifeTable(
			new int[]{0, 1, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 
					70, 75, 80, 85, 90, 95, 100},
					new double[]{
					0.0502,
					0.01273,
					0.01813,
					0.00632,
					0.01167,
					0.04169,
					0.08878,
					0.13015,
					0.14072,
					0.13819,
					0.10376,
					0.09637,
					0.10613,
					0.1262,
					0.16889,
					0.24338,
					0.35132,
					0.49935,
					0.66916,
					0.78066,
					0.84565,
					1});

	public void returnDeathMonth(Agent agent) {
		double chance = Math.random();
		if (agent.getGender() == Settings.GENDER.FEMALE) {
			if (chance < 0.0502) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(1, 12));
				agent.setExpectedLife(1);
				return;
			}
			else if (chance < 0.06473) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(13, 60));
				agent.setExpectedLife(5);
				return;				
			}
			else if (chance < 0.08286) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(61, 120));
				agent.setExpectedLife(10);
				return;								
			}
			else if (chance < 0.08918) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(121, 180));
				agent.setExpectedLife(15);
				return;								
			}
			else if (chance < 0.10085) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(181, 240));
				agent.setExpectedLife(20);
				return;								
			}
			else if (chance < 0.14254) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(241, 300));
				agent.setExpectedLife(25);
				return;								
			}
			else if (chance < 0.23132) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(301, 360));
				agent.setExpectedLife(30);
				return;								
			}			
			else if (chance < 0.59279) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(361, 420));
				agent.setExpectedLife(35);
				return;								
			}			
			else if (chance < 0.73351) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(421, 480));
				agent.setExpectedLife(40);
				return;											
			}			
			else if (chance < 0.8717) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(481, 540));
				agent.setExpectedLife(45);
				return;											
			}			
			else if (chance < 0.97546) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(50);
				return;											
			}
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(50);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(55);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(60);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(65);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(70);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(75);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(80);
				return;											
			}	
			else if (chance < 0.10376) {
				agent.setDeathTick(agent.getBirthTick() + Uniform.staticNextIntFromTo(541, 600));
				agent.setExpectedLife(85);
				return;											
			}	
			else {
				
			}

		}
		else {
			
		}
	}*/
	
	/**
	 * Returns the transmission probability based on Koopman et al. (1994; 1997)
	 */
	public double returnTransmissionProbabilityKoopman (Agent indexAgent, int currentTick) {
		double probability = 1; 		 		 
		if (currentTick - indexAgent.getInfectedTick() <= 3) {
			//early infection
			probability = 0.02;
			return  probability;
		}
		if (currentTick - indexAgent.getInfectedTick() < indexAgent.getInfectedLag()) {
			//middle stage
			probability = 0.0012;
			return probability;			 
		}
		if (currentTick > indexAgent.getInfectedTick() + indexAgent.getInfectedLag()) {
			probability = 0.076;
			return probability;
		}	  
		return 0.0012;
	}

	public double returnTransmissionProbabilityWawerPinkerton(Agent indexAgent) {
		double probability = 0;	 
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.EARLY_HIV) {
			probability = 0.01069;
			return  probability;			
		}
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.EARLY_HIV) {
			probability = 0.00165;
			return probability;			
		}
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.LATE_HIV) {
			probability = 0.00104;
			return probability;			
		}
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.AIDS) {
			probability = 0.0028;
			return probability;			
		}
		return probability;		
	}

	/**
	 * Returns the transmission probability based on Wawer et al. (2006)
	 */
	public double returnTransmissionProbabilityWawer(Agent indexAgent) {
		double probability = 0; 		 
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.EARLY_HIV) {
			probability = 0.0082;
			return  probability;			
		}
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.EARLY_HIV) {
			probability = 0.0015;
			return probability;			
		}
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.LATE_HIV) {
			probability = 0.0007;
			return probability;			
		}
		if (indexAgent.getHealthStatus() == Settings.HEALTH_STATUS.AIDS) {
			probability = 0.0028;
			return probability;			
		}
		return probability;	
	}

	public static void main(String[] args) {
		AidsSpread p = new AidsSpread();
		for (int i = 0; i < 2000; i++){
//			p.returnGamma(i);
//			System.out.println("i: " + i);
//			System.out.print(" tick: " + i);
			p.returnIncidence();
		}
	}
}