package infrastructure;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import display.Node;

/**
 * Clasas to calculate 2-sample KS test
 * @author shah
 *
 */
public class KS {	
	private static final double EPS1 = 0.001;
	private static final double EPS2 = 1.0E-8; 

	private HashMap<Integer, ArrayList<Double>> ksData = new HashMap<Integer, ArrayList<Double>>();				

	public double prob = 0;
	public double diff = 0;

	public void addList(int timeStep, ArrayList<Node> networkList) {
		ArrayList<Double> distribution = new ArrayList<Double>();		
		for (Node node : networkList) {
			distribution.add(new Double(node.getOutNodes().size()));			
		}		
		ksData.put(new Integer(timeStep), distribution);		
	}
	
	public void consecutiveKS() {
		int pajekInterval = Settings.PAJEK_INTERVAL;
		for (int i=50; i <=Settings.NUM_RECORDS; i+=pajekInterval) {
			calculateKS(i-pajekInterval, i);
		}		
	}

	public boolean calculateKS(int timeStep1, int timeStep2) {
		Integer t1 = new Integer(timeStep1);
		Integer t2 = new Integer(timeStep2);

		if (!ksData.containsKey(t1) || !ksData.containsKey(t2)) {
			return false;
		}

		ArrayList<Double> d1 = ksData.get(t1);
		double[] data1 = new double[d1.size()];		
		for (int i=0; i<d1.size(); i++) {
			data1[i] = d1.get(i).doubleValue();
		}
		
		PrintWriter netRecorder;
		String prefix = "d1";
		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".txt")));
			int i = 0;
			for (i=0; i<d1.size(); i++) {
				netRecorder.println(data1[i] + ", ");
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();					
		}	

		ArrayList<Double> d2 = ksData.get(t2);
		double[] data2 = new double[d2.size()];		
		for (int i=0; i<d2.size(); i++) {
			data2[i] = d2.get(i).doubleValue();
		}

		prefix = "d2";
		try {
			netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(""
					+ prefix + ".txt")));
			int i = 0;
			for (i=0; i<d2.size(); i++) {
				netRecorder.println(data2[i] + ", ");
			}
			netRecorder.close();
		} catch (IOException e) {
			e.printStackTrace();					
		}	
		kstwo(data1, data2);
		return true;
	}

	public void kstwo(double[] data1, double[] data2) {
		double d1=0, d2=0, dt, fn1=0,fn2=0;		
		int n1 = data1.length-1;
		int n2 = data2.length-1;

		double en1 = n1; 
		double en2 = n2;

		data1 = sort(data1);
		data2 = sort(data2);		

		int j1=0, j2=0;
		while (j1 <= n1 && j2 <= n2) {
			d1 = data1[j1];
			d2 = data2[j2];
			if (d1 <= d2) {
				fn1 = j1/en1;
				j1++;
			}
			if (d2 <= d1) {
				fn2 = j2/en2;
				j2++;
			}
			dt = Math.abs(fn2-fn1);
			if (dt > diff) {
				diff = dt;
			}			
		}

		double en = Math.sqrt(en1*en2/(en1+en2));
		prob = probKS((en+0.12+0.11/en)*diff);
		System.out.println("Prob: " + prob + " Diff: " + diff);
	}	 

	public double probKS(double alam) {
		double a2=0, fac=2.0, sum=0, term=0, termbf=0;
		a2 = -2*alam*alam;
		for (int j=1; j<100; j++) {
			term = fac*Math.exp((double)a2*j*j);
			sum += term; 
			if (Math.abs(term) < EPS1 * termbf
					|| Math.abs(term) <= EPS2*sum) {
				return sum;
			}
			fac = -fac;
			termbf = Math.abs(term);
		}
		return 1.0;				
	}

	public static void main(String[] args) {
		KS ks = new KS();
		//test
		double[] data1 ={23.4, 30.9, 18.8, 23.0, 21.4, 1, 24.6, 23.8, 24.1, 18.7, 16.3, 20.3, 14.9, 35.4, 21.6, 21.2, 21.0, 15.0, 15.6, 24.0, 34.6, 40.9, 30.7, 24.5, 16.6, 1, 21.7, 1, 23.6, 1, 25.7, 19.3, 46.9, 23.3, 21.8, 33.3, 24.9, 24.4, 1, 19.8, 17.2, 21.5, 25.5, 23.3, 18.6, 22.0, 29.8, 33.3, 1, 21.3, 18.6, 26.8, 19.4, 21.1, 21.2, 20.5, 19.8, 26.3, 39.3, 21.4, 22.6, 1, 35.3, 7.0, 19.3, 21.3, 10.1, 20.2, 1, 36.2, 16.7, 21.1, 39.1, 19.9, 32.1, 23.1, 21.8, 30.4, 19.62, 15.5};
		double[] data2 = {16.5, 1, 22.6, 25.3, 23.7, 1, 23.3, 23.9, 16.2, 23.0, 21.6, 10.8, 12.2, 23.6, 10.1, 24.4, 16.4, 11.7, 17.7, 34.3, 24.3, 18.7, 27.5, 25.8, 22.5, 14.2, 21.7, 1, 31.2, 13.8, 29.7, 23.1, 26.1, 25.1, 23.4, 21.7, 24.4, 13.2, 22.1, 26.7, 22.7, 1, 18.2, 28.7, 29.1, 27.4, 22.3, 13.2, 22.5, 25.0, 1, 6.6, 23.7, 23.5, 17.3, 24.6, 27.8, 29.7, 25.3, 19.9, 18.2, 26.2, 20.4, 23.3, 26.7, 26.0, 1, 25.1, 33.1, 35.0, 25.3, 23.6, 23.2, 20.2, 24.7, 22.6, 39.1, 26.5, 22.7};		
		ks.kstwo(data1, data2);	
	}

	public double getDiff() {
		return diff;
	}

	public double getProb() {
		return prob;
	}

	public HashMap<Integer, ArrayList<Double>> getKsData() {
		return ksData;
	}	

	public double[] sort(double[] a) {
		QuickSort(a, 0, a.length-1);
		return a;
	}

	/**Quicksort based on: 
	 * http://www.koders.com/java/fid18DC52F02D2E4B02E04E6CE32F297C028F485366.aspx */
	private void QuickSort(double a[],int lo0, int hi0) {
		int lo = lo0;
		int hi = hi0;
		double mid;
		if ( hi0 > lo0) {
			//Arbitrarily establishing partition element as the midpoint of the array.			 
			mid = a[ ( lo0 + hi0 ) / 2 ];
			// loop through the array until indices cross
			while( lo <= hi ) {
				/* find the first element that is greater than or equal to
				 * the partition element starting from the left Index.
				 */
				while( ( lo < hi0 ) && ( a[lo] < mid ))  {
					++lo;
				}
				/* find an element that is smaller than or equal to
				 * the partition element starting from the right Index.
				 */
				while( ( hi > lo0 ) && ( a[hi] > mid )) {
					--hi;
				}
				// if the indexes have not crossed, swap
				if( lo <= hi ) {
					double temp = 0; 
					temp = a[lo];
					a[lo] = a[hi];
					a[hi] = temp;
					++lo;
					--hi;
				} 
			}
			/* If the right index has not reached the left side of array
			 * must now sort the left partition.
			 */
			if( lo0 < hi ) {
				QuickSort( a, lo0, hi );
			}
			/* If the left index has not reached the right side of array
			 * must now sort the right partition. */
			if( lo < hi0 ) {
				QuickSort( a, lo, hi0 );
			}
		}		
	}
}