package units;

import java.util.*;

import clubs.*;
import infrastructure.*;
import infrastructure.Settings.GRANTS;
import economics.*;
import cern.jet.random.Gamma;
import cern.jet.random.Normal;
import cern.jet.random.Uniform;
import display.*;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;

/**
 * Class representing an individual person as agent.
 * 
 */
public class Agent implements Comparable {
	/** agent's id ... */
	private int id = -1;
	/** assign 0 -> 'Female'; '1' -> 'Male' ; -1 -> 'unassigned' */
	private int gender = -1;
	/** ID of this household to this individual belongs */
	private int houseHoldID = -1;
	/** for wives only */
	private int maternalHouseholdID = -1;
	/** health status of the agent*/
	private int healthStatus = Settings.HEALTH_STATUS.WELL;
	/** age of the agent*/
	private int age = 0;
	/**
	 * a grant is an agent's property, though it's shared by the entire
	 * household. It's also assumed that an individual is getting exactly one
	 * grant, e.g. if there is a child suffering from HIV we allocate the grant
	 * which is more i.e., the child_grant and when he/she gets old, it
	 * start receiving the hiv grant because the child grant stops
	 */
	private Grant grant;
	private Model model;
	/**Household of the agent */
	private Household household;
	private int maritalStatus = -1, motherID = -1, fatherID = -1,
				spouseID = -1, ageGroup = -1;
	private boolean alive = true, orphan = false;
	/**default education level */
	private String achievedEducation = "none";
	private boolean atSchool = false; 
	/** the health count */
	private int healthCount = 100;
	/** Initial feed status */
	private int feedStatus = Settings.FEED_STATUS.FULL_FED;
	/** is head of the household */
	private boolean head = false;
	/** is single mother */
	private boolean singleMother = false;
	/** is pregnant */
	private boolean pregnant = false;
	/** father id for single mother's casde */
	private int fatherOfMyChildID = -1;

	/**
	 * Last Age: is drawn from a normal distribution with a mean life expectancy
	 * and standard deviation ... both PARAMETERS in the Settings ... SEE
	 * constructor ... needs separate parameter values for males and females
	 */
	private int expectedLife = -1, birthTick = -1, deathTick = -1, 
	infectedTick = -1, infectedAge = -1, pregnancyTick = -1, infectedLag = -1;

	private boolean innovator = false, memberSavings = false,
	pullOutMembership = false, committed = false;
	/** friends list */
	private ArrayList<Agent> friends = new ArrayList<Agent>(),
			invited = new ArrayList<Agent>(), acceptances = new ArrayList<Agent>();
	/** last try for stokvel count */
	private double lastTryStokvel = 0;
	/** number of stokvel tries  */
	private int numStokvelTries = 0;
	public static int INVITE = 0, INVITE2 = 1, ACCEPT = 2, REJECT = 3, STOKVEL = 4;
	/** is memebr of funeral club*/
	private boolean memberFuneralClub = false;
	private int numMigrationTries = Uniform.staticNextIntFromTo(3, 9);
	private int lastMigrationTry = 0;
	/** is fit ofr labor */
	private boolean laborFit = true;
	/** is having piecejob */
	private boolean pieceJob = false;
	/** is migrated */
	private boolean migrated = false;
	private int migratedTick = -1, singleRemittanceLimit = -1;
	private int remittance_Amount = -1;
	/** is visiting home */
	private boolean onVisit = false;	
	private int lastVisit = -1;
	/**
	 * to be replaced by a sigmoid curve
	 */
	private int fertilityAgeLimit = -1;
	/**My current empoloyer*/
	private Household employer = new Household();

	private ArrayList<Message> stovelMessageInbox = new ArrayList<Message>();
	private ArrayList<Message> stovelMessageOutbox = new ArrayList<Message>();

	private ArrayList<Message> sexualPartnershipMessageInbox = new ArrayList<Message>();
	private ArrayList<Message> sexualPartnershipMessageOutbox = new ArrayList<Message>();


	/**
	 * start: the following are regarding the endorsement mech to be implemented
	 * for finding partners
	 */
	private String[] positiveEndorsements = {"same-age", "wealth", "on-job", "quality"};
	private String[] negativeEndorsements = {"old-age", "poor", "jobless"};
	private double base = Random.uniform.nextDoubleFromTo(1.0, 3.0);
//	this is the maximumWeightForEndorsementClasses
	private int numEndorsementClasses = Random.uniform.nextIntFromTo(2, 5);
	private int [] positiveEndosrsementValues = new int [this.positiveEndorsements.length];
	private int [] negativeEndosrsementValues = new int [this.negativeEndorsements.length];

	/**
	 * the following is from my implementation of the marriage model
	 * proposed by Todd and Billari (2003), reported by Heuveline (2003)
	 * Ref: Todd, P.M. and Billari, F.C. (2003), Population-wide marriage patterns by
	 * individual mate-heuristics, in Billari, F.C. and F/urnkranz-Prskawetz, A. (eds.)
	 * Agent-Based Computational Demographic Behavior, New York: Physica-Verlag.	 *
	 */
	private double quality = 0; 
	private double aspirationLevel = 0;
	/** number of months I can wait without a partner*/ 
	private int waitingThreshold = 0;
	/** number of months I have been waiting without a partner*/
	private int waitingTime = 0;
	private int courtShipDuration = 0, maxCourtShipDuration = 0;
	private Agent potentialPartner = null;
	/** Current proposers for sexual relation*/
	private ArrayList<Agent> proposers = new ArrayList<Agent>();
	/** Current sexual partners*/
	private ArrayList<Agent> sexualPartners = new ArrayList<Agent>();
	/** Max sexual partners I can have*/
	private int maxSexualPartners = 0;
	/** Potential female sexual partners*/
	private ArrayList<Agent> potentialFemales = new ArrayList<Agent>();
	/** My offsprings*/
	private ArrayList<Agent> offSprings = new ArrayList<Agent>();
	/** Chance that I get a child grant, if eligible*/
	private boolean wouldGetChildGrant = false;
	/** Chance that I get a disability grant, if eligible*/
	private boolean wouldGetDisabilityGrant = false;
	/** Chance that I get a pension grant, if eligible*/
	private boolean wouldGetPension = false;

	/**
	 * mining stuff
	 */
	private boolean mineTrained = false;
	private boolean mineJob = false;
	private boolean outsider = false;
	private int numMonthsOutsiderArrival = -1;
//	private int maxOutsiderWaitingMonths = Random.uniform.nextIntFromTo(5, 11);
//	private int maxOutsiderWaitingMonths = Random.uniform.nextIntFromTo(1, 5);
	private int maxOutsiderWaitingMonths = Random.uniform.nextIntFromTo(1, 5);
	private int midLag = -1, lateLag = -1;
	/** List of my acquaintances*/
	private ArrayList<Agent> acquaintances = new ArrayList<Agent>();	

	public Agent () {
	}

	public Agent (int _id) {
		this.id = _id;
	}

	/**
	 * for Empirical Households
	 */
	public Agent (int _id, Household _household, Model _model) {
		this.id = _id;
		this.household = _household;
		this.houseHoldID = _household.getId();
		this.model = _model;
		this.healthStatus = Settings.HEALTH_STATUS.WELL;
//		initialize();
	}

	/**
	 * Used for creating coarse-grained household composition
	 * @param _id
	 * @param _gender
	 * @param _ageGroup
	 * @param _healthStatus
	 * @param _household
	 * @param _model
	 */
	public Agent(int _id, int _gender, int _ageGroup, int _healthStatus, Household _household, Model _model) {
		this.id = _id;
		setAgeGroup(_ageGroup);
		this.healthStatus = _healthStatus;
		this.household = _household;
		this.houseHoldID = _household.getId();
		this.gender = _gender;
		this.model = _model;
		switch (this.ageGroup) {
		case Settings.AGE_GROUP.CHILD:
			age = Random.uniform.nextIntFromTo(1, Settings.CHILD_AGE_LIMIT);
			break;
		case Settings.AGE_GROUP.ADULT:
			age = Random.uniform.nextIntFromTo(Settings.CHILD_AGE_LIMIT + 1,
					Settings.ADULT_AGE_LIMIT);
			break;
		case Settings.AGE_GROUP.SENIOR:
			age = Random.uniform.nextIntFromTo(Settings.ADULT_AGE_LIMIT + 1,
					(int) Settings.MEAN_LIFE_EXPECTANCY);
			break;
		default:
			break;
		}
		initialize();
	}
	/**
	 * For outsider agents 
	 */

	public Agent (int _id, Model _model) {
		this.id = _id;
		this.model = _model;
		setOutsider(true);
		//whether skilled or not 
		//magic number ... very small are skilled
		setMineTrained(Math.random() < 0.001 ? true : false);
		this.gender = Settings.GENDER.MALE;
		this.ageGroup = Settings.AGE_GROUP.ADULT;
		this.healthStatus = Settings.HEALTH_STATUS.WELL;
		this.household = new Household();
		this.houseHoldID = -1;
		model.getGui().addAgentNode(this);
		model.updateGraphLayoutAll();
		initializeOutsider();
	}

	/** This is for child who's born during simulation */
	public Agent(int _id, int _gender, int _healthStatus, Household _household,
			Model _model) {
		this.id = _id;
		setAgeGroup(Settings.AGE_GROUP.CHILD);
		this.age = 1;
		this.healthStatus = _healthStatus;
		this.household = _household;
		this.houseHoldID = _household.getId();
		this.model = _model;
		this.gender = _gender;
		initialize();
		initializeFriends();

		model.getGui().addAgentNode(this);
		model.updateGraphLayoutAll();
	}

	/** Initialize settings for Outsider agents*/ 
	public void initializeOutsider() {
		this.age =  (int) Normal.staticNextDouble(30, 10);
		this.expectedLife = (int) model.getExpectedAgeNormal().nextDouble();
		while (this.expectedLife <= this.age) {
			this.expectedLife = (int) model.getExpectedAgeNormal().nextDouble();			
		}		
		if (this.age == this.expectedLife) {
			this.expectedLife += Random.uniform.nextIntFromTo(1, 10);
		}
		model.getAgentMap().put(new Integer(this.id), this);
		this.maritalStatus = Settings.MARITAL_STATUS.SINGLE;
		infectedLag = (int) model.getWeibull().returnWeibull() * 12;
		assignEndorsmentValues();
		initializeFertlityLimit();
		initializeCourtshipSettings();
		updateGrantChances();
	}

	/** Initialize settings for village agents*/
	public void initialize() {
		// CAUTION: this is same for male/females ... need different
		this.expectedLife = (int) model.getExpectedAgeNormal().nextDouble();
		while (this.expectedLife <= this.age) {
			this.expectedLife = (int) model.getExpectedAgeNormal().nextDouble();			
		}		
		if (this.age == this.expectedLife) {
			this.expectedLife += Random.uniform.nextIntFromTo(1, 10);
		}
		this.innovator = Math.random() < Settings.INNOVATOR_PROBABILITY ? true : false;
		if (this.household.getId() != -1) {
			model.getAgentList().add(this);	
		}
		model.getAgentMap().put(new Integer(this.id), this);
//		this.birthTick = (int) model.getTickCount();
		if (model.getTickCount() < 12) {
			this.birthTick = (int) Random.uniform.nextDoubleFromTo(model.getTickCount(),
					model.getTickCount() + 11);
		}
		else {
			this.birthTick = (int) model.getTickCount();
		}	
		grant = new Grant(GRANTS.NO_GRANT, this);
		this.maritalStatus = Settings.MARITAL_STATUS.SINGLE;
		infectedLag = (int) model.getWeibull().returnWeibull() * 12;
		assignEndorsmentValues();
		initializeFertlityLimit();
		initializeCourtshipSettings();
		updateGrantChances();
	}

	/**Update my grant chances */
	private void updateGrantChances() {
//		if (Math.random() < 0.33) {
		//magic number
		if (Math.random() < 0.20) {
			this.wouldGetChildGrant = true;
		}
		this.wouldGetChildGrant = true;
		if (Math.random() < 0.1111) {
			this.wouldGetDisabilityGrant = true;
		}
		if (Math.random() < 0.1666) {
			this.wouldGetPension = true;
		}
	}

	/**Assign fertility age limit */
	private void initializeFertlityLimit() {
		if (this.gender == Settings.GENDER.FEMALE) {
			this.fertilityAgeLimit = (int) model.getFemaleFertilityNormal().nextDouble();
		}
	}

	/**
	 * from the marriage model ... 
	 */
	public void initializeCourtshipSettings() {
		// for outsiders
		if (this.outsider && this.houseHoldID == -1) {
						this.maxSexualPartners = (int) Math.round(model.getOutsiderPsr().nextLogNormal(0.7, 0.9));
			setCourtShipDuration((int) Math.round(model.getOutsiderPsr()
					.nextLogNormal(0.9, 0.7) * 7));
			setAspirationLevel(model.getOutsiderPsr().nextLogNormal(0.0, 1.0) * 10);
			setQuality(model.getOutsiderPsr().nextLogNormal(0.0, 1.0) * 10);
			setWaitingThreshold(Random.uniform.nextIntFromTo(5, 10));
			 			
			this.maxSexualPartners = Random.uniform.nextIntFromTo(1, 2);
			setCourtShipDuration(50);
			setAspirationLevel(Random.uniform.nextIntFromTo(5, 7));
			setQuality(Random.uniform.nextIntFromTo(4, 7));
			setWaitingThreshold(Random.uniform.nextIntFromTo(5, 10));
			return;
		}

//		magic numbers
		setAspirationLevel(model.getPsr().nextLogNormal(0.0, 1.0) * 10);
//		setAspirationLevel(model.getPsr().nextLogNormal(0.0, 0.9));	
//		setAspirationLevel(0);
//		magic numbers
		setQuality(model.getPsr().nextLogNormal(0.0, 1.0) * 10);		
		//magic numbers
		/*		do {
			setQuality(model.getPsr().nextGaussian(7, 3));
		} while (this.quality < 3 || this.quality >= 10);
		 */
		//		magic numbers
//		setWaitingThreshold(Random.uniform.nextIntFromTo(5, 10));
		setWaitingThreshold((int)model.getPsr().nextGaussian(7, 3));
		setMaxSexualPartners();
		do {
			//magic numbers
			this.maxCourtShipDuration = (int) Math.round(model.getCourtShipDuration()
					.nextLogNormal(0.9, 0.7) * 7);
			//magic numbers
		} while (this.maxCourtShipDuration <= 12 || this.maxCourtShipDuration >= 36);

		updateCourtshipThreshold();
	}


	/**
	 * Set Maximum sexual partners
	 *
	 */
	private void setMaxSexualPartners() {
		/*		if (this.gender == Settings.GENDER.FEMALE) {
			do {
				//(0.2, 0.3)
				this.maxSexualPartners = (int) Math.round(model.getPsr().nextLogNormal(0.2, 0.3));				 
			} while (this.maxSexualPartners <= 1 || this.maxSexualPartners >= 3);			
//			} while (this.maxSexualPartners > 3);
		}
		else if (this.gender == Settings.GENDER.MALE) {
			do {
				//(0.4, 0.7)
				this.maxSexualPartners = (int) Math.round(model.getPsr().nextLogNormal(0.4, 0.7));				 
			} while (this.maxSexualPartners <= 1 || this.maxSexualPartners > 5);					

		}
		 */	
		if (this.gender == Settings.GENDER.FEMALE) {
			do {
				//(0.2, 0.3)
				this.maxSexualPartners = (int) Math.round(model.getPsr().nextLogNormal(0.2, 0.3));				 
			} while (this.maxSexualPartners < 0 || this.maxSexualPartners > 3);			
//			} while (this.maxSexualPartners > 3);
		}
		else if (this.gender == Settings.GENDER.MALE) {
			do {
				//(0.4, 0.7)
				this.maxSexualPartners = (int) Math.round(model.getPsr().nextLogNormal(0.4, 0.7));				 
			} while (this.maxSexualPartners < 0 || this.maxSexualPartners >= 5);					

		}

	}

	/**
	 * friendship is dyadic (symmetric) a and there's another starting
	 * assumption that no two friends belong to the same household .. even
	 * though it might be possible later on, i.e. when the hh are accommodated
	 * etc.
	 */
	public void initializeFriends() {
		int initSize = Random.uniform.nextIntFromTo(5, Settings.FRIENDSHIP_UPPER_LIMIT);
		int friends = 0;
		SimUtilities.shuffle(model.getAgentList());
		for (Agent potentialFriend : model.getAgentList()) {
			if (friends == initSize) {
				break;
			}
			if ((potentialFriend.getAgeGroup() == this.ageGroup
					//magic number
					|| Math.abs(potentialFriend.getAge() - this.age) <= 8)
					&& (this.household.getId() != potentialFriend.getHousehold().getId())
					&& (this.id != potentialFriend.getId())
					&& !this.friends.contains(potentialFriend)
					&& !potentialFriend.getFriends().contains(this)
					&& this.friends.size() < Settings.FRIENDSHIP_UPPER_LIMIT
					&& potentialFriend.getFriends().size() < Settings.FRIENDSHIP_UPPER_LIMIT
			) {
				this.friends.add(potentialFriend);
				potentialFriend.getFriends().add(this);

				friends++;					
			}
		}
	}

	public void assignEndorsmentValues() {
		for (int i=0; i<this.positiveEndorsements.length; i++) {
			this.positiveEndosrsementValues[i] = Random.uniform.nextIntFromTo(1, numEndorsementClasses);
		}
		for (int i=0; i<this.negativeEndorsements.length; i++) {
			this.negativeEndosrsementValues[i] = Random.uniform.nextIntFromTo(1, numEndorsementClasses);
		}
	}
	
	/**Increase age */
	public void incrementAge() {
		this.age++;
	}

	/**set life status */
	public void setAlive(boolean _alive) {
		this.alive = _alive;
	}

	/**Am I alive? */
	public boolean isAlive() {
		return this.alive;
	}

	/** assigne education levels - from RM */
	public void education(int currentTick) {		
		if (currentTick % 12 == 0) {
			if (!this.atSchool && (this.age >= 6) && (this.age <= 13)){
				setAtSchool(true);	// start school
				setAchievedEducation(Settings.educationLevels[1]);
			}			
			else if (atSchool){
				switch (age){
				case 13 : setAchievedEducation(Settings.educationLevels[2]); break; // finished primary school
				case 14 : setAchievedEducation(Settings.educationLevels[3]); break; // some secondary school				
				case 16 : setAchievedEducation(Settings.educationLevels[4]); setAtSchool(false); break; // finished secondary school
				default: break;
				}
				// let them finish uni between 22 and 25
				if (this.age > 16 && this.age < 20) {
					//magic number
					int tertiaryFee = 1000;
					if (migrated
							&& (this.achievedEducation.compareTo(Settings.educationLevels[4]) == 0)
							&& this.household.getWealth() > tertiaryFee) {
						setAtSchool(false);
						setAchievedEducation(Settings.educationLevels[5]);						
					}
				}
				else if (this.age >= Random.uniform.nextIntFromTo(21,25)){
					setAtSchool(false);
				}
			}
		}					
	}

	/** Update agents' profile at every step; 
	 * includes ageing, death, pregnancy, health status
	 * and health count
	 */
	public void updateProfile(int currentTick) {		
		if (!isAlive()) {
			return;
		}		

		if (this.deathTick == currentTick) {
			setAlive(false);
			return;
		}

		if (this.gender == Settings.GENDER.FEMALE
				&& this.pregnancyTick >= 0 
				&& this.pregnancyTick < 9) {
			this.pregnancyTick++;
		}			

		if ((currentTick - this.birthTick) % 12 == 0) {
			updateAge();
			updateGrantChances();
		}
		education(currentTick);
		setRemittance_Amount(0);
		setPieceJob(false);
		// update the health status
		if (this.healthCount > 0 ) {
			updateHealthStatus(currentTick);			
		}
		if (hasHIV()) {
			updateHIVHealthStatus(currentTick);	
		}	
		// health is updated every month
		updateHealthCount(currentTick);
		this.laborFit = isLaborFit(currentTick);
		setAlive(true);
		//check death is my implementation for a normal sampled expected life
		checkDeath();
	}

	/*	public void updateHIVHealthStatus(int currentTick) {
		if (currentTick > this.infectedTick + this.infectedLag) {
			fastDecayHealth(currentTick);
		}
	}
	 */

	/**
	 * Decay health of agents having HIV/AIDS late stages
	 */
	public void updateHIVHealthStatus(int currentTick) {
		if (this.healthStatus == Settings.HEALTH_STATUS.LATE_HIV
				|| this.healthStatus == Settings.HEALTH_STATUS.AIDS) {
//		if (this.healthStatus == Settings.HEALTH_STATUS.AIDS) {
			fastDecayHealth(currentTick);
		}
	}

	/** Updates age and health of outsider agents*/
	public void updateOutsiderProfile(int currentTick) {
		if (!isMineJob()) {
			this.numMonthsOutsiderArrival++;	
		}		
		if (!isAlive()) {
			return;
		}	
		if ((currentTick - this.birthTick) % 12 == 0) {
			updateAge();
		}			
		this.setFeedStatus(Settings.FEED_STATUS.FULL_FED);
		updateHealthCount(currentTick);
		if (this.healthCount > 0 ) {
			updateHealthStatus(currentTick);
		}
		if (hasHIV()) {
			updateHIVHealthStatus(currentTick);	
		}	
		this.laborFit = isLaborFit(currentTick);
		setAlive(true);
		checkDeath();
	}

	/**
	 * Post-Validation and Post-Fieldtrip*
	 * Decide whether should I migrate or not 
	 */
//	TODO: Borrow money for migration????
	public boolean shouldMigrate() {			
		// assuming only adults migrate
		if (this.ageGroup == Settings.AGE_GROUP.ADULT
				&& this.household.getWealth() > Settings.MIGRATION.AMOUNT_REQD_MIGRATION
//				&& Math.random() < 0.15
		) {
			return true;
		}
		/*		if (!this.household.getChildren().isEmpty()
				&& household.getChildren().get(0).getFeedStatus() == Settings.FEED_STATUS.HALF_FED
				&& this.household.getWealth() > Settings.MIGRATION.AMOUNT_REQD_MIGRATION
				&& this.feedStatus == Settings.FEED_STATUS.HALF_FED) {
			return true;
		}
		 */
		return false;
	}

	/**Return an acquaintance who has already migrated */
	public Agent returnAcquaintanceCity() {
		//magic number
		int tries = Random.uniform.nextIntFromTo(1, this.friends.size());		

		for (Agent agent : model.getMigratedAgents()) {
			if (tries < 0) {
				break;
			}
			if (this.friends.contains(agent)
					|| this.household.getExtendedFamilyIDList().contains(new Integer(agent.getHouseHoldID()))
					|| (this.household.getFuneralClub().getId() == -1 
							&& this.household.getFuneralClub().getId() 
							== agent.getHousehold().getFuneralClub().getId())) {
				return agent;
			}
			tries--;
		}
		return null;
	}

	/**
	 * this only updates the health count of agents regardless of their age and
	 * sex and HIV ... this is a crude implementation and needs to be worked on
	 * in much detail ... a crucial method
	 */
	public void updateHealthCount(int currentTick) {
		switch (this.feedStatus) {
		case Settings.FEED_STATUS.FULL_FED:
			if (this.healthCount < 98) {
				this.healthCount += Random.uniform.nextIntFromTo(1, 2);
			} else if (this.healthCount >= 100) {
				this.healthCount = 100;
			}
			break;
		case Settings.FEED_STATUS.HALF_FED:
			// magic number
			this.healthCount -= Random.uniform.nextIntFromTo(3, 5);
			break;
		case Settings.FEED_STATUS.NOT_FED:
			// magic number
			this.healthCount -= Random.uniform.nextIntFromTo(5, 7);		
			break;
		default:
			break;
		}		
		//magic number
		if (this.healthCount < 5) {
			this.healthCount = 0;
		}
	}

	/**Update My health stauts */
	public void updateHealthStatus(int currentTick) {
		if (this.infectedTick == 0 && hasHIV()) {
			switch (this.healthStatus) {
			case Settings.HEALTH_STATUS.EARLY_HIV:
				if (currentTick >= this.midLag) {
					this.healthStatus = Settings.HEALTH_STATUS.MIDDLE_HIV;
				}					
				break;
			case Settings.HEALTH_STATUS.MIDDLE_HIV:
				if (currentTick >= this.lateLag) {
					this.healthStatus = Settings.HEALTH_STATUS.LATE_HIV;
				}				
				break;
			case Settings.HEALTH_STATUS.LATE_HIV:
				if (currentTick >= this.infectedLag) {
					this.healthStatus = Settings.HEALTH_STATUS.AIDS;
				}
				break;
			case Settings.HEALTH_STATUS.AIDS:
				break;
			default: break;			
			}
			return;
		}

//		Wawer
		if (hasHIV()) {						
			if (currentTick - this.infectedTick <= 3) {
				this.healthStatus = Settings.HEALTH_STATUS.EARLY_HIV;
				return;
			}
			else if ((currentTick - this.infectedAge > 3)
					&& (currentTick - this.infectedTick <= 15)) {
				this.healthStatus = Settings.HEALTH_STATUS.MIDDLE_HIV;
				return; 
			}
			else if (currentTick - this.infectedTick < this.infectedLag) {
				this.healthStatus = Settings.HEALTH_STATUS.LATE_HIV;
				return;
			}
			else {
				this.healthStatus = Settings.HEALTH_STATUS.AIDS;
				return;
			}			
		}


		//Pinkerton
		/*		if (hasHIV()) {						
			if (currentTick - this.infectedTick < 10) {
				this.healthStatus = Settings.HEALTH_STATUS.EARLY_HIV;
				return;
			}
			else if ((currentTick - this.infectedAge >= 10)
					&& (currentTick - this.infectedTick < 20)) {
				this.healthStatus = Settings.HEALTH_STATUS.MIDDLE_HIV;
				return; 
			}
			else if (currentTick - this.infectedTick < this.infectedLag) {
				this.healthStatus = Settings.HEALTH_STATUS.LATE_HIV;
				return;
			}
			else {
				this.healthStatus = Settings.HEALTH_STATUS.AIDS;
				return;
			}			
		} */


		switch (this.ageGroup) {
		case Settings.AGE_GROUP.SENIOR:
			// magic number ... needs qualitative a/c
			if (this.healthCount > 60) {
				// we will use old age sickness for small labor etc.(SJA Aug 11)
				setHealthStatus(Settings.HEALTH_STATUS.OLD_AGE_SICKNESS);
			} else {
				setHealthStatus(Settings.HEALTH_STATUS.DISABLE);
			}
			break;
		case Settings.AGE_GROUP.ADULT:
		case Settings.AGE_GROUP.CHILD:
			//magic number
			if (this.healthCount > 25) {
				setHealthStatus(Settings.HEALTH_STATUS.WELL);
			} else {
				// we will use illness for migration and other
				// local jobs later (SJA Aug 11)
				setHealthStatus(Settings.HEALTH_STATUS.ILL);
			}
			break;
		default:
			break;
		}	
	}

	/**Decay health based on Sigmoid function*/
	public void fastDecayHealth(int currentTick) {
		int o = healthCount; 
//		this.healthCount *= model.getSig().valueofFunction(this.age - this.infectedAge);
//		this.healthCount *= model.getSig().valueofFunction(this.deathTick - currentTick);
		this.healthCount *= model.getSig().valueofFunction(currentTick - this.lateLag);
		model.print("Agent-" +id + " prev-healthCount: " + o + " - new healthCount " + healthCount);
	}

	/**Update my age and age groups*/
	public void updateAge() {
		this.age++;
		boolean ageGroupChanged = false;
		if ((this.ageGroup == Settings.AGE_GROUP.CHILD)
				&& (this.age > Settings.CHILD_AGE_LIMIT)
				&& (this.age <= Settings.ADULT_AGE_LIMIT)) {
			setAgeGroup(Settings.AGE_GROUP.ADULT);
			ageGroupChanged = true;
		} else if ((this.ageGroup == Settings.AGE_GROUP.ADULT)
				&& (this.age > Settings.ADULT_AGE_LIMIT)) {
			setAgeGroup(Settings.AGE_GROUP.SENIOR);
			ageGroupChanged = true;
		}
		else {
			ageGroupChanged = false;
		}
		if (ageGroupChanged) {
			//			Household household = (Household) model.getHouseMap().get(
			//					new Integer(this.houseHoldID));
			if (model.getHouseholdList().contains(household)) {
				household.updateAgentAgeGroup(this, this.ageGroup);
				if (this.ageGroup == Settings.AGE_GROUP.ADULT) {
					initializeFriends();
					model.getGui().addAgentNode(this);
					model.updateGraphLayoutAll();					
				}
			}
		}
		updateCourtshipThreshold();
	}

	/**
	 * there could be several reasons for a person dying ... so that's why we've
	 * got a separate method - to kill ... - old age == Need Normal Dist central
	 * at 50 ~ life expectancy rate: South Africa - hiv/aids - lack of nutrition
	 * or illness
	 */
	public void checkDeath() {
		if (this.age >= this.expectedLife || this.healthCount <= 0) {			
			setAlive(false);
		} else {
			setAlive(true);
		}
	}

	/**
	 * Get the grandparent's household, if it exists
	 * @return
	 */
	public Household getGPHousehold() {
		Household gpHousehold = new Household();
		Agent father = null, mother = null;
		if (this.fatherID == -1 && this.motherID == -1) {
			return gpHousehold;
		}		
		if (this.fatherID != -1) {
			father = model.getAgentMap().get(new Integer(this.fatherID));	
		}
		if (this.motherID != -1) {
			mother = model.getAgentMap().get(new Integer(this.motherID));	
		}								
		gpHousehold = retrieveGPHousehold(father);
		if (gpHousehold.getId() != -1 && gpHousehold != null) {
			return gpHousehold;
		} 
		else {
			gpHousehold = retrieveGPHousehold(mother);
		}
		return gpHousehold;
	}

	public Household getElderSiblingOrAuntHousehold() {
		Household relativeHousehold = new Household();		
		//checking for elder sibling in other household
		for (Agent agent : model.getAgentList()) {
			Agent father = model.getAgentMap().get(new Integer(this.fatherID));
			Agent mother = model.getAgentMap().get(new Integer(this.motherID));			
			if (agent.isAlive()
					&& agent.getAgeGroup() != Settings.AGE_GROUP.CHILD
					&& agent.getGender() == Settings.GENDER.FEMALE
					&& !(agent.getMotherID() == -1 && agent.getFatherID() == -1)) {
				if (father != null) {
					if ((agent.getFatherID() == this.fatherID && this.fatherID != -1)
							|| (agent.getFatherID() == father.getFatherID() 
									&& agent.getFatherID() != -1)) {
						relativeHousehold = agent.getHousehold();
						break;
					}
				}
				if (mother != null) {
					if (agent.getMotherID() == this.motherID 
							|| agent.getMotherID() == mother.getMotherID()) {
						relativeHousehold = agent.getHousehold();
						break;
					}
				}
			}
		}		
		return relativeHousehold;
	}

	/**
	 * SJA: Aug 27, 2007
	 * now introducing the fertility rate and 
	 * declining of age into fertility with age
	 * TODO: we need to introduce a sigmoid for a gradual decline in the
	 * fertility of women
	 * Reference: www.bcm.edu/news/item.cfm?newsID=66 
	 * TODO: to get a proper reference 
	 */
	public boolean fertility(int currentTick) {
		if (this.age >= this.fertilityAgeLimit) {
			return false;
		}		
		if (this.pregnancyTick != -1 
				|| !hasHIV()) {
			return true;
		}
		/*
		 * however, HIV-infected women gave birth only 74 percent as often as
		 * uninfected women in the 7.5 to 5.0-year period before AIDS diagnosis,
		 * 55 percent as often in the 5.0 to 2.5 quarter, and 45 percent as
		 * often in the 2.5 to 0 quarter.
		 * http://www.jhsph.edu/publichealthnews/press_releases/PR_2000/HIV_fertility.html
		 */
		int difference = currentTick - this.infectedTick;
		if (difference <= 12) {
			return true;
		} else if (difference <= 18) {
			return Math.random() < 0.75 ? true : false;
		} else if (difference <= 24) {
			return Math.random() < 0.55 ? true : false;
		} else {
			return Math.random() < 0.40 ? true : false;
		}
	}

	/**
	 * Get grandparent's household: maternal or paternal depending upon the 
	 * parent 
	 * @param parent
	 * @return
	 */
	public Household retrieveGPHousehold(Agent parent) {
		Household household = new Household();
		if (parent == null) {
			return household;	
		}
		if (parent.getId() == -1) {
			return household;
		}

		Agent grandParent = null;
		grandParent = model.getAgentMap().get(new Integer(parent.getFatherID()));

		if (grandParent != null
				&& grandParent.isAlive() 
				&& grandParent.getId() != -1
				&& grandParent.getHousehold().getId() != this.houseHoldID) {
			household = grandParent.getHousehold();
		}
		return household;
	}

	/**
	 * following meeting with Gina at Aberdeen ... usually, it's 60% male and
	 * 20% female "(and(agent (isAlive TRUE)(ageGroup ~" +
	 * Settings.AGE_GROUP.CHILD + ")(isRemittance FALSE)(OBJECT ?agent))" +
	 * "(agent (ageGroup " + Settings.AGE_GROUP.SENIOR + ")(healthStatus " +
	 * Settings.HEALTH_STATUS.DISABLE + ")(OBJECT ?agent)))" +
	 * 
	 */
	public boolean isLaborFit(int currentTick) {
		/*		if ((this.ageGroup == Settings.AGE_GROUP.SENIOR 
				&& this.healthStatus != Settings.HEALTH_STATUS.DISABLE)
				|| (this.ageGroup == Settings.AGE_GROUP.ADULT 
						&& (this.healthStatus != Settings.HEALTH_STATUS.HIV 
								|| (this.healthStatus == Settings.HEALTH_STATUS.HIV 
										&& this.infectedTick + Settings.INFECTED_LAG <= (int) model.getTickCount())))) {
			return true;
		}
		return false;
		 */		
		if ((this.ageGroup == Settings.AGE_GROUP.SENIOR 
				&& this.healthStatus != Settings.HEALTH_STATUS.DISABLE)
				|| (this.ageGroup == Settings.AGE_GROUP.ADULT						
						&& this.healthStatus == Settings.HEALTH_STATUS.ILL)
						|| this.ageGroup == Settings.AGE_GROUP.CHILD){
			return false;
		}
				
		if (this.healthStatus == Settings.HEALTH_STATUS.AIDS) {
			return false;
		}

		return true;
	}

	/**
	 * currently, it's not the youngest ... but picking the last male in the
	 * family ... the last one to marry ...
	 */
	public boolean isYoungestSon() {
		//		Household household = (Household) model.getHouseMap().get(
		//				new Integer(this.houseHoldID));
		//		if (!model.getHouseholdList().contains(household)) {
		//			return false;
		//		}
		int numMales = 0;
		for (Agent agent : this.household.getAdults()) {
			if (agent.getGender() == Settings.GENDER.MALE) {
				numMales++;
			}
		}
		return numMales == 1 ? true : false;
	}

	/**
	 * those who're married/widowed would send the major
	 * proportion of the remittance magic number: 70-80% of the remittance to
	 * their respective households ... the remainder of the money is assumed to
	 * be contributed towards the agent's accommodating hh in the city. - those
	 * who're unmarried send money, i.e. 70-80% but stop sending money back
	 * after sometime [magic number]
	 */
	/*	public int returnMyRemittance() {
		Double amt = new Double(0);
		//now using left-skewed distribution as suggested by Anton...here, the Gamma Distribution
		this.remittance_Amount = (int) Gamma.staticNextDouble(Settings.REMITTANCE.SKEWED_REMITTANCE, 1.25);
		//Using Normal distribution
//		this.remittance_Amount = (int) Normal.staticNextDouble(
//		Settings.REMITTANCE.MEAN, Settings.REMITTANCE.STD_DEV);
		if (this.maritalStatus == Settings.MARITAL_STATUS.SINGLE
				&& this.migratedTick - (int) this.model.getTickCount() < this.singleRemittanceLimit) {
			amt = Random.uniform.nextIntFromTo(65, 80) * 0.01
	 * this.remittance_Amount;
		} else {
			// magic number
			// amt = Random.uniform.nextIntFromTo(55, 70)*0.01*this.remittance_Amount;
			amt = Random.uniform.nextIntFromTo(45, 60) * 0.01 * this.remittance_Amount;
		}		
		return amt.intValue();
	}*/

	/**
	 * again magic assumptions ... but this time, the remittance 
	 * corresponds to the education level of the agents
	 * TODO: 1. agents on migrationn should have tenures/durations for jobs ... also ... their income should remain fixed ... 
	 */
	public int returnMyRemittance() {
		//TODO: 1. agents on migrationn should have tenures/durations for jobs ... also ... their income should remain fixed ... 
		this.remittance_Amount = 0;
		int index = 0;
		for (int i=0; i<Settings.educationLevels.length; i++) {
			if (this.achievedEducation.equals(Settings.educationLevels[i])) {
				index = i;
				break;
			}
		}

		//some-primary/primary, some-secondary/secondary, tertiary 
//		public static int[] migrantRemittanceScales = {300, 400, 750, 1000};
		// {"none", "some-primary", "primary", "some-secondary", "secondary", "tertiary"};
		switch(index) {
		//none/some-primary/primary
		case 0: case 1: case 2:
			this.remittance_Amount = Uniform.staticNextIntFromTo(300, 400);
			break;		
		case 3: case 4: 
			this.remittance_Amount = Uniform.staticNextIntFromTo(400, 750);
			break;
		case 5: this.remittance_Amount = Uniform.staticNextIntFromTo(750, 1000);
		default: break;
		}

		if (this.maritalStatus == Settings.MARITAL_STATUS.SINGLE
				&& this.migratedTick - (int) this.model.getTickCount() < this.singleRemittanceLimit) {
			this.remittance_Amount = (int) (Random.uniform.nextIntFromTo(65, 80) * 0.01
					* this.remittance_Amount);
		} else {
			// magic number
			this.remittance_Amount = (int) (Random.uniform.nextIntFromTo(45, 60) * 0.01 
					* this.remittance_Amount);
		}		
		return this.remittance_Amount;
	}

	/** Return my renumeration from mine */
	public int returnMyMineJobEarning() {
		return model.getMine().returnEmployeeRenumeration(this);
	}

	/** Return my renumeration from piecework */
	public int returnMyPieceWorkEarning() {		
		if (this.employer.getId() != -1) {
//			now using left-skewed distribution as suggested by Anton...here, the Gamma Distribution
			this.remittance_Amount = (int) Gamma.staticNextDouble(Settings.REMITTANCE.SKEWED_PIECE_WORK, 1.25);
			//Using Normal distribution
			//magic number
//			this.remittance_Amount = (int) Normal.staticNextDouble(75, 15);
			this.employer.deductWealth(this.remittance_Amount);
			Double amt = (double) this.remittance_Amount;
			return amt.intValue();
		}
		else {
			return 0;
		}
	}

	/**
	 * from RADAR: a hungry person would ask their neighbors (in our case, it's
	 * friends) not more than two days in a row. this 25% chance is just a Q&D
	 * solution to making the daily 'two days in a row' to our monthly scale ...
	 * this needs further work
	 */
	public boolean askFood(Agent friend) {
		// magic number
		return Math.random() < 0.20 ? true : false;
	}

	/** Should I feed my friend */
	public boolean feedFriend(Agent agent) {
		if (this.feedStatus == Settings.FEED_STATUS.NOT_FED) {
			return false;
		}
		//		Household household = (Household) model.getHouseMap().get(
		//				new Integer(this.houseHoldID));
		/*
		 * actually the cost should be lesser. this is because: 1. we're at the
		 * monthly timescale 2. a food once cooked can be shared by many so the
		 * cost is lesser. we therefore need further on this.
		 */
		if (this.household.getWealth() >= Settings.MIN_MONEY_TO_FEED) {
			this.household.deductWealth((double) Settings.FEED_STATUS.HALF_FED);
			return true;
		}
		return false;
	}

	/**Add me to the clinic
	 * 
	 */
	public void hivTreatment(int currentTick) {
		model.getClinic().addPatient(this, currentTick);
	}

	/** My attempt to migrate at this time */
	public void attemptMigration(int currentTick) {
		if (	isMigrated() 
				&& !laborFit 
				|| currentTick - lastMigrationTry < numMigrationTries
				|| isMineJob()) {
			return;
		}
		//TODO: also, not everyone who has migration gets the job ... need
		//to incorporate the delay in getting the job...
		boolean chanceMigration = true;
		if (this.gender == Settings.GENDER.FEMALE) {
			//magic number
			chanceMigration = Math.random() < 0.05 ? true : false;
		}
		//magic number
		else if (this.gender == Settings.GENDER.MALE) {
//			chanceMigration = Math.random() < 0.15 ? true : false;
			chanceMigration = Math.random() < 0.10 ? true : false;
		}		

		if (!migrated 
				&& shouldMigrate() 
				&& chanceMigration) {	

			if (this.head && !household.updateHead(this)) {
				return;
			}				

			Agent acquaintance = returnAcquaintanceCity();			
			if (acquaintance != null || Math.random() < 0.10) {
				migrate(currentTick);
			}
		}							

		if (migrated) {			
			household.getCurrentMembers().remove(this);
			this.pullOutMembership = true;
		}
		else {
			this.lastMigrationTry = currentTick;
		}
	}

	/** I migrate at this time */
	public void migrate(int currentTick) {
		this.household.deductWealth(Settings.MIGRATION.AMOUNT_REQD_MIGRATION);
		if (!model.getMigratedAgents().contains(this)) {
			model.getMigratedAgents().add(this);
		}
		this.migrated = true;
		this.migratedTick = currentTick;
		// magic number
		this.singleRemittanceLimit = Random.uniform.nextIntFromTo(Settings.MIGRATION.SINGLE_REMITTANCE_LIMIT, 48);

		//SJA 2008
		/*		this.clearAllSexualPartners();
		if (this.maritalStatus == Settings.MARITAL_STATUS.MARRIED) {
			Agent spouse = (Agent) model.getAgentMap().get(spouseID);
			if (spouse != null) {
				this.sexualPartners.add(spouse);
				spouse.getSexualPartners().add(this);
			}									
		}
		 */
	}

	/** Get piecwork for this agent*/ 
	public void getPieceWork(int currentTick) {
		if (!migrated
				&& laborFit
				&& (this.gender == Settings.GENDER.FEMALE && Math.random() < 0.25)
				|| (this.gender == Settings.GENDER.MALE && Math
						.random() < 0.45)) {
			this.pieceJob = true;
		}
	}

	/** Apply for piecwork for this agent*/
	public void applyForPieceWork(int currentTick) {
		for (Household household : model.getCurrentEmployers()) {
			//magic number
			if (household.getAaplicants().size() < 15) {
				household.getAaplicants().add(this);
			}
		}
	}

	/**Send invitation to join stokvel*/ 
	public void sendInvitation() {
		this.invited.clear();
		this.acceptances.clear();
		HouseholdNode myHouseholdNode = null;
		if (!model.getGui().getHouseholdNodeMap().containsKey(this.household.getId())) {
			this.household.printhouseholdComposition();
		}
		myHouseholdNode = (HouseholdNode) model.getGui().getHouseholdNodeMap()
		.get(new Integer(this.household.getId())).node;

		for (Agent friend : this.friends) {
			if (friend.isAlive()
					&& !this.invited.contains(friend)) {
				Message message = new Message(this, friend, 
						Settings.MESSAGE_TYPE.CLUB_INVITATION);
				stovelMessageOutbox.add(message);
				this.invited.add(friend);				
			}
		}

		for (Agent friend : this.friends) {
			for (Agent friendOfFriend : friend.getFriends()) {
				if (this.id != friendOfFriend.getId()
						&& friendOfFriend.isAlive()
						&& !this.invited.contains(friendOfFriend)) {
					Message message = new Message(this, friend, 
							Settings.MESSAGE_TYPE.CLUB_INVITATION);
					stovelMessageOutbox.add(message);
					this.invited.add(friend);							
				}
			}
		}
		ArrayList<HouseholdNode> neighbours = myHouseholdNode.getToNodes();
		SimUtilities.shuffle(neighbours);
		for (HouseholdNode neighbourHH : neighbours) {
			Agent neighbourHead = (Agent) neighbourHH.getHousehold().getHead();
			if (neighbourHead != null
					&& !this.invited.contains(neighbourHead)
					&& neighbourHead.isAlive()) {
				Message message = new Message(this, neighbourHead, 
						Settings.MESSAGE_TYPE.CLUB_INVITATION);
				stovelMessageOutbox.add(message);
				this.invited.add(neighbourHead);				
			}
		}
		ArrayList<Agent> temp = new ArrayList<Agent>(this.invited); 
		for (Agent agent : temp) {
			//explicitly strict gendered-specific
			if (agent.getGender() != this.gender) {
				this.invited.remove(agent);
			}
		}
		SimUtilities.shuffle(this.invited);
		SimUtilities.shuffle(stovelMessageOutbox);

	}
	
	public void sendInvitation2() {
		this.invited.clear();
		this.acceptances.clear();
		HouseholdNode myNode = null;
		if (!model.getGui().getHouseholdNodeMap().containsKey(this.houseHoldID)) {
			this.household.printhouseholdComposition();
		}

		myNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(new Integer(this.houseHoldID)).node;
		ArrayList<HouseholdNode> neighbours = myNode.getToNodes();
		SimUtilities.shuffle(neighbours);
		for (HouseholdNode node : neighbours) {
			Agent neighbourHead = node.getHousehold().getHead();			
			if (neighbourHead != null 
					&& neighbourHead.getGender() == Settings.GENDER.FEMALE
					&& !this.invited.contains(neighbourHead)
					&& neighbourHead.isAlive()
			) {

			}
		}				

		ArrayList<Agent> temp = new ArrayList<Agent>(this.friends);
		for (Agent friend : temp) {
			if (!model.getAgentList().contains(friend)
					|| !friend.isAlive()) {
				this.friends.remove(friend);								
			}
			else if (
					!this.invited.contains(friend)
					&& friend.getGender() == Settings.GENDER.FEMALE
					&& friend.isHead()
					&& friend.isAlive()
			) {
				Message message = new Message(this, friend, 
						Settings.MESSAGE_TYPE.CLUB_INVITATION);
				stovelMessageOutbox.add(message);
				this.invited.add(friend);
			}
		}

		HouseholdNode myExtendedFamilyNode = (HouseholdNode) model.getGui()
		.getExtendedFamilyNodeMap().get(new Integer(this.houseHoldID)).node;
		ArrayList<HouseholdNode> kins = new ArrayList<HouseholdNode>(myExtendedFamilyNode.getToNodes());

		SimUtilities.shuffle(kins);
		for (HouseholdNode node : kins) {
			Agent kinHead = node.getHousehold().getHead();			
			if (	kinHead != null 
					&& kinHead.getGender() == Settings.GENDER.FEMALE
					&& !this.invited.contains(kinHead)
					&& kinHead.isAlive()
			) {
				Message message = new Message(this, kinHead, 
						Settings.MESSAGE_TYPE.CLUB_INVITATION);
				stovelMessageOutbox.add(message);
				this.invited.add(kinHead);				
			}
		}			
	}

	/**Return stokvel candidates for the new stokvel */
	protected ArrayList<Agent> getStokvelCandidates() {
		HouseholdNode myNode = null;
		ArrayList<Agent> candidates = new ArrayList<Agent>();

		if (model.getGui().getHouseholdNodeMap().get(
				new Integer(this.houseHoldID)) == null) {
			// return empty list
			return candidates;
		}

		myNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(
				new Integer(this.houseHoldID)).node;
		// fetch all social neighbours who are female and household heads
		ArrayList<HouseholdNode> neighbours = myNode.getToNodes();
		// SimUtilities.shuffle(neighbours);
		for (HouseholdNode node : neighbours) {
			Agent neighbourHead = node.getHousehold().getHead();
			// this is an strict assumption ... the gender one ...
			if (neighbourHead != null
					&& neighbourHead.getGender() == Settings.GENDER.FEMALE
					&& neighbourHead.getAgeGroup() != Settings.AGE_GROUP.CHILD
					&& neighbourHead.isAlive()) {					
				candidates.add(neighbourHead);
			}
		}
		//need to clean my friends list
		// fetch all friends who are female and household heads.
		ArrayList<Agent> temp = new ArrayList<Agent>(this.friends);
		for (Agent friend : temp) {
			if (!model.getAgentList().contains(friend)) {
				// if friend is dead ... no longer exists
				// CAUTION: BUGGY
				this.friends.remove(friend);
			} else if ((friend.getGender() == Settings.GENDER.FEMALE)
					&& (!candidates.contains(friend)) && friend.isHead()
					&& friend.getAgeGroup() != Settings.AGE_GROUP.CHILD
					&& friend.isAlive()) {
				candidates.add(friend);
			} 			
		}

		SimUtilities.shuffle(model.getHouseholdList());
		int random = Uniform.staticNextIntFromTo(1, 3);
		for (Household hh : model.getHouseholdList()) {
			if (random < 0) {
				break;
			}
			if (this.household.getId() != hh.getId()) {
				Agent head = hh.getHead();
				if (head != null) {
					if (!candidates.contains(head) && this.id != head.getId()
							&& head.getGender() == Settings.GENDER.FEMALE
							&& head.getAgeGroup() != Settings.AGE_GROUP.CHILD
							&& head.isAlive()) {
						candidates.add(head);							
					}								
				}
			}
		}
		return candidates;
	}		

	/**I'm the moderator/facilitator; creating the stokvel */
	public void createStokvel() {
		SavingsClub club = new SavingsClub(this, this.acceptances, model);
		this.acceptances.clear();
		model.createSavingsClub(club);
	}
	
	/**Return my father id*/
	public int getFatherID() {
		return fatherID;
	}
	/**Set my father id*/
	public void setFatherID(int fatherID) {
		this.fatherID = fatherID;
	}
	/**Return my mother id*/
	public int getMotherID() {
		return motherID;
	}
	/**Set my mother id*/
	public void setMotherID(int motherID) {
		this.motherID = motherID;
	}
	/**Return my marital status*/
	public int getMaritalStatus() {
		return maritalStatus;
	}
	/** Set my marital status*/
	public void setMaritalStatus(int maritalStatus) {
		this.maritalStatus = maritalStatus;
	}
	/**Is this one an orphan*/
	public boolean isOrphan() {
		return orphan;
	}
	/**make orphan*/
	public void setOrphan(boolean isOrphan) {
		this.orphan = isOrphan;
	}
	/**Return my spouse id*/
	public int getSpouseID() {
		return spouseID;
	}
	/** Sets my spouse id*/
	public void setSpouseID(int spouseID) {
		this.spouseID = spouseID;
	}
	/**Return my expected life*/
	public int getExpectedLife() {
		return expectedLife;
	}
	/**Set my expected life*/
	public void setExpectedLife(int expectedLife) {
		this.expectedLife = expectedLife;
	}
	/** this method used for monthly scale */
	public void setInfectedTick(int currentTick) {
		this.infectedTick = currentTick;
		this.infectedAge = this.age;
	}
	/** Get my infected */
	public int getInfectedTick() {
		return this.infectedTick;
	}
	/**Return feed status */
	public int getFeedStatus() {
		return feedStatus;
	}
	/**Set feed status */
	public void setFeedStatus(int feedStatus) {
		this.feedStatus = feedStatus;
	}
	/**Return my remittance */
	public int getRemittance_Amount() {
		return remittance_Amount;
	}
	/**Set my remittance */
	public void setRemittance_Amount(int remittance_Amount) {
		this.remittance_Amount = remittance_Amount;
	}
	/**Am I innovator*/
	public boolean isInnovator() {
		return innovator;
	}
	/**Make me an innovator*/
	public void setInnovator(boolean isInnovator) {
		this.innovator = isInnovator;
	}

	public boolean isMemberSavings() {
		return memberSavings;
	}

	public void setMemberSavings(boolean _isMemberSavings) {
		this.memberSavings = _isMemberSavings;
		setCommitted(false);
	}

	/**Am I a head */
	public boolean isHead() {
		return head;
	}
	/**Make me household head*/
	public void setHead(boolean isHead) {		
		this.head = isHead;
		if ((int) model.getTickCount() >= 1
				&& this.gender == Settings.GENDER.FEMALE) { 
			model.getGui().addSavingsNode(this);
			model.updateGraphLayoutAll();
		}
		setIsMemberFuneralClub(this.household.isMemberFuneralClub());
	}
	/**Get my friends */
	public ArrayList<Agent> getFriends() {
		return friends;
	}
	/**set my friends */
	public void setFriends(ArrayList<Agent> friends) {
		this.friends = friends;
	}
	/**Am I pulling out from stokvel */
	public boolean isPullOutMembership() {
		return pullOutMembership;
	}
	/**Set me as pulling out from stokvel */
	public void setPullOutMembership(boolean pullOutMembership) {
		this.pullOutMembership = pullOutMembership;
	}

	public void setSavingsPool(int sum) {
		household.addWealth((double) sum);
	}

	public void flushMembership() {
		this.memberSavings = false;
		if (!this.pullOutMembership) {
			/*
			 * if we were to contribute, but the club ceased to be viable we
			 * deposit back the money (which was deducted from the household
			 * pool, back to the household
			 */
			setSavingsPool(Settings.STOKVEL_AMOUNT);
		}
		this.pullOutMembership = false;
	}

	public ArrayList<Agent> getAcceptances() {
		return acceptances;
	}

	public void setAcceptances(ArrayList<Agent> acceptances) {
		this.acceptances = acceptances;
	}

	public boolean isCommitted() {
		return this.committed;
	}

	public void setCommitted(boolean _committed) {
		this.committed = _committed;
	}

	public int getNumStokvelTries() {
		return this.numStokvelTries;
	}

	public void setNumStokvelTries(int n) {
		this.numStokvelTries = n;
	}

	public void incNumStokvelTries() {
		setNumStokvelTries(this.numStokvelTries + 1);
	}

	public void decNumStokvelTries() {
		setNumStokvelTries(this.numStokvelTries - 1);
	}
	/**Find me a funeral club */
	public void findFuneralClub() {
		//		Household household = (Household) model.getHouseMap().get(
		//				new Integer(this.houseHoldID));
		//		if (model.getHouseholdList().contains(household)) {
		FuneralClub club = (FuneralClub) this.household.findFuneralClub();
		if (club != null) {
			// we join the club
			if (club.handleRequestMembership()) {
				this.memberFuneralClub = true;
				club.addMember(this.household);
				this.household.setFuneralClub(club);
			}
		}
		//		}
	}
	/**Get my funeral club */
	public boolean getMemberFuneralClub() {
		return memberFuneralClub;
	}

	public void setIsMemberFuneralClub(boolean _isMemberFuneralClub) {
		this.memberFuneralClub = _isMemberFuneralClub;
	}
	/** Have I migrated*/
	public boolean isMigrated() {
		return migrated;
	}
	/** Set me as migrated*/
	public void setMigrated(boolean isMigrated) {
		this.migrated = isMigrated;
	}
	
	public int getMigratedTick() {
		return migratedTick;
	}
	/** Get my pre-marriage household id*/
	public int getMaternalHouseholdID() {
		return maternalHouseholdID;
	}
	/** Set my pre-marriage household id*/
	public void setMaternalHouseholdID(int maternalHouseholdID) {
		this.maternalHouseholdID = maternalHouseholdID;
	}
	/** Get my household*/
	public Household getHousehold() {
		return household;
	}
	/** Set my household*/
	public void setHousehold(Household household) {
		this.household = household;
	}
	/** Make new friend*/
	public void makeFriend(Agent friend) {
		if (model.getGui().getAgentFriendNodeMap().get(new Integer(this.id)) == null) {
			model.print("i am null");
		}
		AgentNode myNode = model.getGui().getAgentFriendNodeMap().get(new Integer(this.id)).node;
		AgentNode friendNode = model.getGui().getAgentFriendNodeMap().get(new Integer(friend.getId())).node;		

		if (!myNode.checkFriendshipLimit(friendNode)) {
			return;
		}
		if (!this.friends.contains(friend)) {
			this.friends.add(friend);	
		}
		if (!friend.getFriends().contains(this)) {
			friend.getFriends().add(this);			
		}		
		myNode.makeEdgeToFrom(friendNode);
	}
	/**Remove all agents as my friends */
	public void clearAllFriends() {
		/*for (Agent friend : this.friends) {
			if (friend.getFriends().contains(this)) {
				friend.getFriends().remove(this);
			}
		}
		this.friends.clear();
		 */
		ArrayList<Agent> temp = new ArrayList<Agent>(this.friends);
		for (Agent friend : temp) {
			removeFriend(friend);
		}
		this.friends.clear();
	}
	/**Remove all my sexual partners*/
	public void clearAllSexualPartners() {
		for (Agent sexualPartner : this.sexualPartners) {
			sexualPartner.getSexualPartners().remove(this);
		}
		this.sexualPartners.clear();
	}
	/**Update my friendship network gui */
	public void updateFriendshipGui() {
		model.getGui().makeFriendsLinks(this);
	}
	/** Get my household*/
	public int getPregnancyTick() {
		return pregnancyTick;
	}
	/** Set my household*/
	public void setPregnancyTick(int pregnancyTick) {
		this.pregnancyTick = pregnancyTick;
	}

	public void stokvelTries(int currentTick){
		if (currentTick - this.lastTryStokvel > 6) { // wait 6 months to
			// try again
			this.setCommitted(false);
			this.setNumStokvelTries(0);
			this.lastTryStokvel = currentTick;
		}
	}
	/** Am I labor fir*/
	public boolean isLaborFit() {
		return laborFit;
	}
	/** Set my fitness status*/
	public void setLaborFit(boolean laborFit) {
		this.laborFit = laborFit;
	}
	/** Am I on piecejob*/
	public boolean isPieceJob() {
		return pieceJob;
	}

	public void setPieceJob(boolean isPieceJob) {
		this.pieceJob = isPieceJob;
	}
	/** Set my age*/
	public void setAge(int _age) {
		age = _age;
	}
	/** Get my age*/
	public int getAge() {
		return this.age;
	}

	/**Get my grant value */
	public int getGrantValue() {
		return this.grant.getType();
	}

	public Grant getGrant() {
		return this.grant;
	}

	public void setGrant(Grant _grant) {
		this.grant = _grant;
	}

	public void setGrant(int grantType) {
		this.grant = new Grant(grantType, this);
	}

	/**Get my age group */
	public int getAgeGroup() {
		return ageGroup;
	}
	/**Set my age group */
	public void setAgeGroup(int _ageGroup) {
		this.ageGroup = _ageGroup;
	}
	/**Get my gender */
	public int getGender() {
		return gender;
	}
	/**Set my age group */
	public void setGender(int gender) {
		this.gender = gender;
	}
	/**Get my health count */
	public int getHealthCount() {
		return healthCount;
	}
	/** Set my health count */
	public void setHealthCount(int _healthCount) {
		this.healthCount = _healthCount;
	}
	/**Set my health status */
	public void setHealthStatus(int _healthStatus) {
		this.healthStatus = _healthStatus;
	}
	/**Get my health status */
	public int getHealthStatus() {
		return this.healthStatus;
	}
	/**Get my health count */
	public int getHouseHoldID() {
		return houseHoldID;
	}
	/**Set my health count */
	public void setHouseHoldID(int houseHoldID) {
		this.houseHoldID = houseHoldID;
	}
	/**Get my Id */
	public int getId() {
		return id;
	}
	/**Set my Id */
	public void setId(int id) {
		this.id = id;
	}
	/**Print my current profile */
	public void printMyCurrentProfile() {
		System.out.println("Tick: + " + model.getTickCount()
				+ " .. I am agent: " + this.id + ". My household is: "
				+ this.getHouseHoldID() + " my age is: " + this.age
				+ " marital status: " + maritalStatus + ". On migration: "
				+ migrated + " is household head: " + head
				+ " has health count: " + this.healthCount
				+ " has health status: " + this.healthStatus
				+ " has expected life: " + this.expectedLife + " has gender: "
				+ this.gender + " has last feed status" + this.feedStatus);			

	}

	public Household getEmployer() {
		return employer;
	}

	public void setEmployer(Household employer) {
		this.employer = employer;
	}
	/**Get my infected lag */
	public int getInfectedLag() {
		return infectedLag;
	}

	/**
	 * Processes stokvel messages inbox
	 *
	 */
	public void processStokvelInbox() {		
		for (Message message : this.stovelMessageInbox) {
			handleMessage(message);
		}		
		this.stovelMessageInbox.clear();
		if (!this.stovelMessageOutbox.isEmpty()) {
			model.getStokvelMessageOutbox().addAll(this.stovelMessageOutbox);
			this.stovelMessageOutbox.clear();
		}
	}
	/**
	 * Processes stokvel messages outbox
	 *
	 */
	public void processStokvelOutbox() {		
		if (sendStokvelInvitation()) {
			sendInvitation();		
		}		
	}

	/**
	 * Process my proposals and messages for forming sexual relationship
	 * @param females
	 */
	public void processSexualPartnershipOutbox(ArrayList<Agent> females) {
		ArrayList<Agent> temp = new ArrayList<Agent>();
		if (this.outsider) {
			potentialFemales = findRandomFemales();
			temp = new ArrayList<Agent>(potentialFemales);
			if (!temp.isEmpty()) {
				for (Agent female : temp) {
					if (!shouldSendOfferOutsider(female)) {
						potentialFemales.remove(female);
					}
				}
				sendCourtShipInvitation();
			}

		}
		else {
			potentialFemales = findSuitableFemalePartners(females);
			temp = new ArrayList<Agent>(potentialFemales);
			if (!temp.isEmpty()) {
				for (Agent female : temp) {
					if (!shouldSendOffer2(female)) {
						potentialFemales.remove(female);
					}
				}
				sendCourtShipInvitation();
			}
		}		
	}

	/**
	 * Send courtship offer to suitable females
	 *
	 */
	public void sendCourtShipInvitation () {
		for (Agent female : this.potentialFemales) {
			Message message = new Message(this, female,	Settings.COURTSHIP_MESSAGE.COURTSHIP_OFFER);
			this.sexualPartnershipMessageOutbox.add(message);												
		}
	}

	/**
	 * Process my proposals and messages for forming sexual relationship
	 *
	 */
	public void processSexualPartnershipInbox() {
		SimUtilities.shuffle(this.sexualPartnershipMessageInbox);
		for (Message message : this.sexualPartnershipMessageInbox) {
			handleSexualPartnershipMessage(message);
		}		
		this.sexualPartnershipMessageInbox.clear();
		if (this.gender == Settings.GENDER.FEMALE
				&& !this.proposers.isEmpty()
				&& this.potentialPartner == null) {
			tryBestProposer();
/*			Agent bestSuitor = (Agent) chooseMaleSexualPartner();
			sendCourtshipAcceptnce(bestSuitor);
			this.proposers.remove(bestSuitor);
*/		}
		else if (this.gender == Settings.GENDER.MALE
				&& !this.potentialFemales.isEmpty()
				&& this.potentialPartner == null){
			tryBestFemale();
		}

		if (!this.sexualPartnershipMessageOutbox.isEmpty()) {
			model.getPartnershipMessageOutbox().addAll(this.sexualPartnershipMessageOutbox);
			this.sexualPartnershipMessageOutbox.clear();
		}
	}

	/**
	 * Handle messages related to formation of sexual partnerships
	 * @param message
	 */
	public void handleSexualPartnershipMessage(Message message) {
		switch (message.getMessageType()) {
		case Settings.COURTSHIP_MESSAGE.NO_MESSAGE:
			break;
		case Settings.COURTSHIP_MESSAGE.COURTSHIP_OFFER:
			handleCourtshipOffer(message);
			break;		
		case Settings.COURTSHIP_MESSAGE.ACCEPTANCE:
			handleCourshipAcceptance(message);
			break;			
		case Settings.COURTSHIP_MESSAGE.REJECTION:
			handleCourshipRejection(message);
			break;
		default: break;
		}
	}

	/** handle offer for courtship */
	public void handleCourtshipOffer(Message message) {
		Agent proposer = (Agent) message.getSender();
		this.proposers.add(proposer);
		if (!isAspirationFit(proposer)) {
			sendCourtshipRejection(proposer);
		}
		else {
			this.proposers.add(proposer);
		}
	}

	/**Assign aspiration to the partners */
	public void assignAspiration() {
		if (this.sexualPartners.size() == 0
				|| (this.maritalStatus == Settings.MARITAL_STATUS.MARRIED
						&& this.sexualPartners.size() == 1)) {
			if (this.gender == Settings.GENDER.FEMALE) {
				if (!this.potentialFemales.isEmpty()) {
					Collections.sort(this.potentialFemales);
					this.aspirationLevel = this.potentialFemales.get(0).getQuality();
				}
				else {
					this.aspirationLevel = this.potentialPartner.getQuality();
				}
			}
			else {
				if (!this.proposers.isEmpty()) {
					Collections.sort(this.proposers);
					this.aspirationLevel = this.potentialFemales.get(0).getQuality();
				}
				else {
					this.aspirationLevel = this.potentialPartner.getQuality();
				}				
			}		
		}
		else {
			potentialPartner.increaseAspiration();
			this.increaseAspiration();
		}
	}
	
	/**Find the best male suitor */
	public void tryBestProposer() {
		Collections.sort(this.proposers);
		int lastIndex = this.proposers.size() - 1;
		sendCourtshipAcceptnce(this.proposers.get(lastIndex));
		this.proposers.remove(lastIndex);		
	}

	/**Find the best female potential partner*/
	public void tryBestFemale() {
		Collections.sort(this.potentialFemales); 
		int lastIndex = this.potentialFemales.size() - 1;
		sendCourtshipAcceptnce(this.potentialFemales.get(lastIndex));
		this.potentialFemales.remove(lastIndex);
	}

	/** send offer for courtship */
	public void sendCourtshipAcceptnce(Agent proposer) {
		Message message = new Message (this, proposer, Settings.COURTSHIP_MESSAGE.ACCEPTANCE);
		this.sexualPartnershipMessageOutbox.add(message);
	}

	/** send rejection for courtship offer */
	public void sendCourtshipRejection (Agent proposer) {
		Message message = new Message (this, proposer, Settings.COURTSHIP_MESSAGE.REJECTION);
		this.sexualPartnershipMessageOutbox.add(message);
	}
	/** handle acceptance for courtship offer */
	public void handleCourshipAcceptance(Message message) {	
		if (this.gender == Settings.GENDER.FEMALE) {
			this.potentialPartner = (Agent) message.getSender();
			this.potentialPartner.setPotentialPartner(this);			
			for (Agent proposer : this.proposers) {
				sendCourtshipRejection(proposer);
			}
			for (Agent female : potentialPartner.getPotentialFemales()) {
				sendCourtshipRejection(female);
			}
			this.proposers.clear();
			potentialPartner.getPotentialFemales().clear();			
			/*this.assignAspiration();
			this.potentialPartner.assignAspiration();*/
			this.increaseAspiration();
			this.potentialPartner.increaseAspiration();
		}
		else if (this.gender == Settings.GENDER.MALE){
			Agent female = (Agent) message.getSender();
			this.potentialPartner = female;
			female.setPotentialPartner(this);			
			for (Agent proposer : female.getProposers()) {
				sendCourtshipRejection(proposer);
			}
			for (Agent agent : this.potentialFemales) {
				sendCourtshipRejection(agent);
			}
			this.potentialFemales.clear();
			female.getProposers().clear();
//			the suites were generated using the below commented code
			female.increaseAspiration();
			this.increaseAspiration();

			/*		female.assignAspiration();
			this.assignAspiration();*/
		}
	}
	/** handle rejection for courtship offer */
	public void handleCourshipRejection(Message message) {
		if (this.gender == Settings.GENDER.MALE) {
			this.potentialFemales.remove(message.getSender());
		}
		else {
			this.proposers.remove(message.getSender());
		}
		decreaseAspiration();
	}
	/** send invitation to join stokvel */
	public boolean sendStokvelInvitation() {
		/*
		 * this one involves the participation from all males ...
		 */
		if (!this.innovator 
				|| memberSavings
				|| this.ageGroup != Settings.AGE_GROUP.ADULT
		){
			return false;
		}

		boolean sendInvitation = false;
		if (this.gender == Settings.GENDER.FEMALE) {
			if (this.maritalStatus != Settings.MARITAL_STATUS.SINGLE 
					|| !this.household.getChildren().isEmpty()) {
				sendInvitation = true;
			}					
		}
		else {
			if ((this.maritalStatus == Settings.MARITAL_STATUS.SINGLE
					&& (this.pieceJob || this.mineJob))
					|| this.maritalStatus == Settings.MARITAL_STATUS.MARRIED){
				sendInvitation = true;
			}
		}		
		return sendInvitation;
	}
	/**Check if I can join a stokvel*/
	public boolean isSavingsPossible() {
		//now check whether I have the money to start a stokvel myself
		double wealth = household.getWealth();
		if (wealth < Settings.STOKVEL_AMOUNT) {
			return false;
		}		
		return true;
	}
	/**Handle stokvel related message*/
	public void handleMessage(Message message) {
		switch (message.getMessageType()) {
		case Settings.MESSAGE_TYPE.NO_MESSAGE:
			break;
		case Settings.MESSAGE_TYPE.CLUB_INVITATION:
			handleIntivation(message);
			break;		
		case Settings.MESSAGE_TYPE.ACCEPTANCE:
			handleAcceptance(message);
			break;			
		case Settings.MESSAGE_TYPE.REJECTION:
			handleRejection(message);
			break;
		default: break;
		}				
	}
	/**Handle stokvel invitation*/
	public void handleIntivation(Message message) {
		Agent sender = (Agent) message.getSender();		
		if (!isSavingsPossible() || memberSavings || committed){
			sendRejection(new Message(this, sender, Settings.MESSAGE_TYPE.REJECTION));
		}
		else {
			//the sender of the original message is now the receiver
			committed = true;
			sendAcceptance(new Message(this, sender, Settings.MESSAGE_TYPE.ACCEPTANCE));
		}		
	}	
	/** send acceptance for stokvel invitation*/
	public void sendAcceptance(Message message) {		
		stovelMessageOutbox.add(message);
	}
	/** handle acceptance for stokvel invitation*/
	public void handleAcceptance(Message message) {
		Agent sender = (Agent) message.getSender();
		acceptances.add(sender);
		invited.remove(sender);

		//-1 because one is us
		if (invited.isEmpty() && (acceptances.size() >= Settings.MIN_SAVINGS_MEMBERS-1)) {
			this.acceptances.add(this);
		}				
	}

	/**
	 * Check if I have an endogamic relation 
	 */
	public boolean endogamicRelation(Agent agent) {
		return isNuclearFamilyRelation(agent) && isUncleOrAunty(agent) ? true : false; 
	}

	/**Check if I have a nuclear family relation with this agent
	 * */
	public boolean isNuclearFamilyRelation(Agent agent) {
		//not a parent
		if (this.motherID == agent.getId() || this.fatherID == agent.getId()) {
			return true;
		}
		//not a sibling
		else if (this.motherID == agent.getMotherID()
				&& this.fatherID == agent.getFatherID()
				&& (this.motherID != -1 && this.fatherID != -1)) {
			return true;
		}
		return false;
	}
	/**Check if this is my uncle or aunty */
	public boolean isUncleOrAunty(Agent agent) {
		Agent father = model.getAgentMap().get(new Integer(this.fatherID));
		Agent mother = model.getAgentMap().get(new Integer(this.motherID));		

		if ((father != null
				&& mother != null
				&& 	father.getFatherID() != -1 && father.getMotherID() != -1)
				&& (agent.getFatherID() != -1 && agent.getMotherID() != -1) 
				&& (mother.getFatherID() != -1 && mother.getMotherID() != -1)
				&& ((father.getFatherID() == agent.getFatherID() 
						|| father.getMotherID() == agent.getMotherID())
						||
						(mother.getFatherID() == agent.getFatherID()
								|| mother.getMotherID() == agent.getMotherID()))) {
			return true;
		}		
		return false;
	}
	/** send rejection for stokvel invitation*/
	public void sendRejection(Message message) {
		stovelMessageOutbox.add(message);
	}
	/** handle rejection for stokvel invitation*/
	public void handleRejection(Message message) {	
		Agent sender = (Agent) message.getSender();
		invited.remove(sender);
	}
	
	public ArrayList<Message> getStovelMessageInbox() {
		return stovelMessageInbox;
	}

	public void setStovelMessageInbox(ArrayList<Message> inbox) {
		this.stovelMessageInbox = inbox;
	}

	public ArrayList<Message> getStovelMessageOutbox() {
		return stovelMessageOutbox;
	}

	public void setStovelMessageOutbox(ArrayList<Message> outbox) {
		this.stovelMessageOutbox = outbox;
	}
	/**Get my achieved education status */
	public String getAchievedEducation() {
		return achievedEducation;
	}
	/**Set my achieved education status */
	public void setAchievedEducation(String educationLevel) {
		this.achievedEducation = educationLevel;
	}
	/**Am I at school */
	public boolean isAtSchool() {
		return atSchool;
	}

	public void setAtSchool(boolean atSchool) {
		this.atSchool = atSchool;
	}
	/**Get fertility age limit */
	public int getFertilityAgeLimit() {
		return fertilityAgeLimit;
	}
	/**Set fertility age limit */
	public void setFertilityAgeLimit(int fertilityAge) {
		this.fertilityAgeLimit = fertilityAge;
	}

	public double getBase() {
		return base;
	}

	public void setBase(double base) {
		this.base = base;
	}

	public int getNumEndorsementClasses() {
		return numEndorsementClasses;
	}

	public void setNumEndorsementClasses(
			int maximumWeightForEndorsementClasses) {
		this.numEndorsementClasses = maximumWeightForEndorsementClasses;
	}

	public String[] getNegativeEndorsements() {
		return negativeEndorsements;
	}

	public String[] getPositiveEndorsements() {
		return positiveEndorsements;
	}

	public double calculateEndorsement(Agent agent) {
		//positive endorsements
		int[] posEndorsements = new int[this.positiveEndorsements.length];
		int[] negEndorsements = new int[this.negativeEndorsements.length];

		for (int i=0; i<this.positiveEndorsements.length; i++) {
			posEndorsements[i] = 0;
		}

		for (int i=0; i<this.negativeEndorsements.length; i++) {
			negEndorsements[i] = 0;
		}

		//magic number
		if (Math.abs(this.age - agent.getAge()) < 5) {
//			posEndorsements[returnIndexOf("same-age", 1)] = 1; 
			posEndorsements[0] = 1;
		}
		if (agent.getHousehold().getWealth() > Settings.RICH_HOUSEHOLD_CRITERIA) {
//			posEndorsements[returnIndexOf("wealth", 1)] = 1;
			posEndorsements[1] = 1;
		}		

		boolean onJob = false;
		if (agent.returnMyPieceWorkEarning() > 0 
				|| agent.returnMyRemittance() > 0
				//SJA Sept 17
				|| agent.isMineJob()) {			
//			posEndorsements[returnIndexOf("on-job", 1)] = 1;
			posEndorsements[2] = 1;
			onJob = true;
		}
		/* to be copied from marriage project
		if (this.aspiration < agent.getBaseQuality()) {
//		posEndorsements[returnIndexOf("baseQuality", 1)] = 1;
 		posEndorsements[3] = 1;
		}			
		 */

		//magic number
		if (agent.getAge() - this.age > 15) {
//			negEndorsements[returnIndexOf("old-age", -1)] = 1; 
			negEndorsements[0] = 1;
		}
//		magic number
		if (agent.getHousehold().getWealth() < 500) {
//			negEndorsements[returnIndexOf("poor", -1)] = 1;
			negEndorsements[1] = 1;
		}
		if (!onJob) {
//			negEndorsements[returnIndexOf("jobless", -1)] = 1;
			negEndorsements[2] = 1;
		}

		double positiveEndorsementValue = 0, negativeEndorsementValue = 0;
		int endValue = 0;
		for (int i=0; i<this.positiveEndorsements.length; i++) {
			endValue = positiveEndosrsementValues[i]; 
			positiveEndorsementValue += (Math.signum(endValue)* Math.pow(base, Math.abs(endValue))
					* posEndorsements[i]);			
		}
		for (int i=0; i<this.negativeEndorsements.length; i++) {
			endValue = negativeEndosrsementValues[i]; 
			negativeEndorsementValue += (Math.signum(endValue)* Math.pow(base, Math.abs(endValue))
					* negEndorsements[i]);		
		}
		double net = positiveEndorsementValue - negativeEndorsementValue;
		return net;
	}

	public int returnIndexOf(String str, int endorsementType) {
		if (endorsementType == 1) {
			for (int i=0; i<this.positiveEndorsements.length; i++) {
				if (this.positiveEndorsements[i].compareTo(str) == 0) {
					return i;
				}
			}
		}
		else if (endorsementType == -1) {
			for (int i=0; i<this.negativeEndorsements.length; i++) {
				if (this.negativeEndorsements[i].compareTo(str) == 0) {
					return i;
				}
			}
		}
		return -1;
	}

	public double getAspirationLevel() {
		return aspirationLevel;
	}

	public void setAspirationLevel(double aspirationLevel) {
		this.aspirationLevel = aspirationLevel;
	}

	public double getQuality() {
		return quality;
	}

	public void setQuality(double baseQuality) {
		this.quality = baseQuality;
	}

	public int getCourtShipDuration() {
		return courtShipDuration;
	}

	public void setCourtShipDuration(int courtShipDuration) {
		this.courtShipDuration = courtShipDuration;
	}

	public int getWaitingThreshold() {
		return waitingThreshold;
	}

	public void setWaitingThreshold(int waitingThreshold) {
		this.waitingThreshold = waitingThreshold;
	}

	public int getWaitingTime() {
		return waitingTime;
	}

	public void setWaitingTime(int waitingTime) {
		this.waitingTime = waitingTime;
	}

	public Agent getPotentialPartner() {
		return potentialPartner;
	}

	public void setPotentialPartner(Agent potentialPartner) {
		this.potentialPartner = potentialPartner;
	}

	public ArrayList<Agent> getProposers() {
		return proposers;
	}

	public boolean isAspirationFit(Agent potentialPartner) {	
		if (this.aspirationLevel <= potentialPartner.getQuality()) {
			return true; 
		}
		else {
			return false;
		}
//		if (this.partner == null) {
//		if (this.aspirationLevel <= potentialPartner.getBaseQuality()) {
//		return true;
//		}
//		else {
//		return false;
//		}					
//		}
//		else {
//		if (this.getPartner().getBaseQuality() < potentialPartner.getBaseQuality()) {
//		return true;
//		}
//		else {
//		return false;
//		}			
//		}
		/*		if (this.sexualPartners.isEmpty()) {
			if (this.aspirationLevel <= potentialPartner.getBaseQuality()) {
				flag = true;
			}
		}		
		else {			
			for (Agent partner : this.sexualPartners) {
				if (partner.getBaseQuality() < potentialPartner.getBaseQuality()) {
					flag = true;
					break;
				}
			}
		}
		return flag;*/
	}

	/**Should send offer to this potential partner */
	public boolean shouldSendOffer(Agent potentialPartner) {
		if (	potentialPartner.endogamicRelation(this)
				|| potentialPartner.getSexualPartners().contains(this)
				|| this.endogamicRelation(potentialPartner) 
				|| potentialPartner.endogamicRelation(this)
		) {
			return false;
		}

		int potentialPartnerAge = potentialPartner.getAge();
		if (this.age >= 16 && this.age <= 24) {
			if (isAspirationFit(potentialPartner)
					&& Math.abs(this.age - potentialPartnerAge) <= 5) {
				return true;
			}
		}
		else if (this.age >= 25 && this.age <= 37) {
			if (isAspirationFit(potentialPartner)
					&& this.age - potentialPartnerAge <= 9) {
				return true;
			}
		}		
		else if (this.age > 37) {
			if (isAspirationFit(potentialPartner)
					|| this.age - potentialPartnerAge > 9) {
				return true;
			}
		}

		return false;

		/*if (isAspirationFit(potentialPartner)) {
			return true;
		}
		else {
			return false;
		}*/

	}
	public boolean shouldSendOffer2(Agent potentialPartner) {		
		if (	potentialPartner.endogamicRelation(this)
				|| potentialPartner.getSexualPartners().contains(this)
				|| this.endogamicRelation(potentialPartner) 
				|| potentialPartner.endogamicRelation(this)
		) {
			return false;
		}		

		if (potentialPartner.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED) {
			return false;
		}

		if (isAspirationFit(potentialPartner)) {
			return true;
		}
		return false;
		  
/*		int potentialPartnerAge = potentialPartner.getAge();

		if (this.age >= 16 && this.age <= 24) {
			if (isAspirationFit(potentialPartner)
					//magic number
					&& Math.abs(this.age - potentialPartnerAge) <= 3) {
				return true;
			}
		}
		else if (this.age >= 25 && this.age <= 37) {
			if (isAspirationFit(potentialPartner)
					//magic number
					&& this.age - potentialPartnerAge <= 7) {
				return true;
			}
		}		
		else if (this.age > 37) {
			if (isAspirationFit(potentialPartner)
					//magic number
					|| this.age - potentialPartnerAge > 9) {
				return true;
			}
		}
		return false;
*/	}

	/**Should send offer to this potential partner - called  if I am an outsider agent*/
	public boolean shouldSendOfferOutsider(Agent potentialPartner) {
		if (potentialPartner.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED
				&& this.maritalStatus == Settings.MARITAL_STATUS.MARRIED
				&& this.sexualPartners.contains(potentialPartner)
				&& this.age < potentialPartner.getAge() + 3				
		) {
			return false;
		}
		return true;
	}

	/**Find the best endorsded suitor for me. */
	public Agent findBestEndorsedSuitor() {
		if (this.proposers.isEmpty()) {
			return null;
		}		
		int size = this.proposers.size();		
		double[] endorsedValues = new double[size];	
		for (int i=0; i<size; i++) {
			endorsedValues[i] = calculateEndorsement(this.proposers.get(i));			
		}		
		//she might just put a threshold ... minium and reject one or all ...
		//depends upon the age ...
		//TODO: that
		if (size == 1) {
			return this.proposers.get(0);
		}

		int maxIndex = 0;
		for (int i=0; i<size; i++) {
			if (endorsedValues[maxIndex] < endorsedValues[i]) {
				maxIndex = i;
			}
		}		
		return this.proposers.get(maxIndex);
	}


	public Agent chooseMaleSexualPartner2() {
		ArrayList<Agent> temp = new ArrayList<Agent>(this.proposers);

		for (Agent agent : temp) {
			if (this.sexualPartners.contains(agent)
					|| this.spouseID == agent.getId()) {
				this.proposers.remove(agent);
			}
		}

		SimUtilities.shuffle(this.proposers);

		Agent malePartner = null;

		if (!this.proposers.isEmpty()) {
			malePartner = this.proposers.get(0);
		}
		return malePartner;
	}

	/**Choose the best male partner for me*/
	public Agent chooseMaleSexualPartner() {
		ArrayList<Agent> temp = new ArrayList<Agent>(this.proposers);
		for (Agent agent : temp) {
			if (this.sexualPartners.contains(agent)
					|| this.spouseID == agent.getId()) {
				this.proposers.remove(agent);
			}		
		}

		SimUtilities.shuffle(this.proposers);
		Agent malePartner = null;

		if (this.age > 29) {
			malePartner = findBestEndorsedSuitor();
			if (malePartner != null && malePartner.getAge() > this.age) {
				return malePartner;
			}
			else {
				return null;
			}
		}
		for (Agent proposer : this.proposers) {
			int proposerAge = proposer.getAge();
			if (this.age >= 16 && this.age <= 23) {
				if (proposer.isMineJob()
						&& this.maritalStatus == Settings.MARITAL_STATUS.SINGLE) {
					malePartner = proposer;
				}				
				if (isAspirationFit(proposer)
						&& this.maritalStatus == Settings.MARITAL_STATUS.SINGLE
						&& Math.abs(this.age - proposerAge) <= 11) {
					malePartner = proposer;
					break;
				}

			}
			else if (this.age > 23 && this.age <= 29) {
				if (isAspirationFit(proposer)
						&& proposer.getMaritalStatus() != Settings.MARITAL_STATUS.MARRIED
						&& (proposer.isPieceJob() || proposer.isMigrated() || proposer.pieceJob)
						&& this.maritalStatus != Settings.MARITAL_STATUS.MARRIED) {
					malePartner = proposer;
					break;
				}
			}		
		}

		//SJA Feb 07 2008

		if (malePartner == null) {
			SimUtilities.shuffle(this.proposers);
			//HACK
			for (Agent proposer : this.proposers) {
				if (isAspirationFit(proposer)) {
					return proposer;
				}	
			}
		}
		return malePartner;

	}
	/**Break my courtship with this partner*/
	public void courtshipBroken(Agent partner) {		
		this.waitingTime = 0;		
		if (this.sexualPartners.contains(partner)) {
			this.sexualPartners.remove(partner);	
		}
	}

	/**Update my courtship duration with age */
	public void updateCourtshipThreshold() {
		this.courtShipDuration = (int) (this.maxCourtShipDuration * (1-((double)this.age/100)));
	}

	/**Update my aspiration level (Heuveline et al. 2003)*/
	public void updateAspirationLevel() {
		if (this.sexualPartners.isEmpty()) {
			waitingTime++;
			if (this.waitingTime > this.waitingThreshold) {
				decreaseAspiration();	
			}			
		}
		else for (Agent partner : this.sexualPartners) {
			if (this.aspirationLevel <= partner.getQuality()) {
				increaseAspiration();
				break;
			}
		}
	}

	/**Find suitabl femal partners for me. */
	public ArrayList<Agent> findSuitableFemalePartners(ArrayList<Agent> availableFemales) {
		setPotentialPartner(null);
		ArrayList<Agent> potentialPartners = new ArrayList<Agent>();
		ArrayList<Agent> temp = new ArrayList<Agent>();

		if (Math.random() < 0.15) {		
			temp = findRandomFemales();
		}
		else {
			temp = returnFemaleFriends();
		}

//		temp = returnFemaleFriends();

		for (Agent agent : temp) {
			if (!potentialPartners.contains(agent)) {
				potentialPartners.add(agent);
			}
		}		
		temp.clear();		
//		temp = returnFemaleAcquaintances();

		for (Agent agent : temp) {
			if (!potentialPartners.contains(agent)) {
				potentialPartners.add(agent);
			}
		}
		SimUtilities.shuffle(potentialPartners);
		temp = new ArrayList<Agent>(potentialPartners);
		for (Agent female : temp) {
			if (this.sexualPartners.contains(potentialPartners)
					|| female.getSexualPartners().contains(this)
					|| (this.spouseID != -1 && this.spouseID != female.getId())
					|| !availableFemales.contains(female)					
			) {
				potentialPartners.remove(female);
			}
		}		
		return potentialPartners;
	}

	/**
	 * now here, we must implement assortative mixing and age specific rules
	 * depending upon the men's preferences
	 */

	public ArrayList<Agent> findRandomFemales() {
		ArrayList<Agent> temp = new ArrayList<Agent>();
		SimUtilities.shuffle(model.getAgentList());
		for (Agent female : model.getAgentList()) {
			if (female.getGender() == Settings.GENDER.FEMALE
					&& !this.sexualPartners.contains(female)
					&& !female.getSexualPartners().contains(this)
					//magic number
					&& Math.random() < 0.15					 
					&& female.getAgeGroup() == Settings.AGE_GROUP.ADULT) {					
				temp.add(female);
			}
		}
		return temp;
	}

	/**Decrease aspiration */
	public void decreaseAspiration() {
		/*		this.aspirationLevel -= 0.1;*/
		this.aspirationLevel--;
		if (this.aspirationLevel < 0) {
			this.aspirationLevel = 0;
		}

	}

	/**Inmcrease aspiration */
	public void increaseAspiration() {
		this.aspirationLevel++;
//		this.aspirationLevel += 0.1;
		/*		if (this.aspirationLevel >= 10) {
			this.aspirationLevel = 10;
		}		
		 */	
	}
	/**Get my birth month*/
	public int getBirthTick() {
		return birthTick;
	}
	/**Set my birth month*/
	public void setBirthTick(int birthTick) {
		this.birthTick = birthTick;
	}
	/**Get my sexual partenrs list*/
	public ArrayList<Agent> getSexualPartners() {
		return sexualPartners;
	}
	/**Set my sexual partenrs list*/
	public void setSexualPartners(ArrayList<Agent> sexualPartners) {
		this.sexualPartners = sexualPartners;
	}
	/**Get my sexual partenrs list*/
	public int getMaxSexualPartners() {
		return maxSexualPartners;
	}
	/**Set my sexual partenrs list*/
	public void setMaxSexualPartners(int maxSexualPartners) {
		this.maxSexualPartners = maxSexualPartners;
	}
	/**Am I mine trained*/
	public boolean isMineTrained() {
		return mineTrained;
	}

	public void setMineTrained(boolean mineTrained) {
		this.mineTrained = mineTrained;
	}
	/**Am I on mine job*/
	public boolean isMineJob() {
		return mineJob;
	}

	public void setMineJob(boolean mineJob) {
		this.mineJob = mineJob;
	}
	/**Am I an outsider*/
	public boolean isOutsider() {
		return outsider;
	}
	/**Set I am an outsider*/
	public void setOutsider(boolean outsider) {
		this.outsider = outsider;
	}

	public int getNumMonthsOutsiderArrival() {
		return numMonthsOutsiderArrival;
	}

	public void setNumMonthsOutsiderArrival(int numMonthsOutsiderArrival) {
		this.numMonthsOutsiderArrival = numMonthsOutsiderArrival;
	}

	public int getMaxOutsiderWaitingMonths() {
		return maxOutsiderWaitingMonths;
	}


	/*	public int compareTo(Object anotherAgent) throws ClassCastException {        
		double anotherAgentsAge = ((Agent) anotherAgent).getAge();
		if (this.age > anotherAgentsAge) {
			return 1;
		} else if (this.age < anotherAgentsAge) {
			return -1;
		} else {
			return 0;
		}
	}
	 */

	/**Compare the agent based on quality */
	public int compareTo(Object anotherAgent) throws ClassCastException {        
		double anotherQuality = ((Agent) anotherAgent).getQuality();
		if (this.quality > anotherQuality) {
			return 1;
		} else if (this.quality < anotherQuality) {
			return -1;
		} else {
			return 0;
		}
	}

	public boolean isOnVisit() {
		return onVisit;
	}

	public void setOnVisit(boolean onVisit) {
		this.onVisit = onVisit;
	}

	/**
	 * It is assumed that a migrant agent sends the remittances every month
	 * through someone. This is being weakly confirmed by the villagers though
	 * it is likely that the remittances may not be that conveniently accessible 
	 * to the migrants' households. 
	 */
	public void visitHome(int currentTick) {
//		int firstRand = Uniform.staticNextIntFromTo(5, 9);
		int firstRand = Uniform.staticNextIntFromTo(2, 5);
//		int secRand = Uniform.staticNextIntFromTo(3, 7);
		int secRand = Uniform.staticNextIntFromTo(3, 5);

		if (	 Math.random() < 0.50
				//at least 4 months have passed
				&& this.migratedTick + firstRand > currentTick
				&& this.lastVisit + secRand > currentTick
		) {
			this.setLastVisit(currentTick);
			this.setOnVisit(true);
			this.household.getCurrentMembers().add(this);
		}		
	}

	public int getLastVisit() {
		return lastVisit;
	}

	public void setLastVisit(int lastVisit) {
		this.lastVisit = lastVisit;
	}

	/**
	 *  break up with all current sexual partners except 
	 *  the spouse
	 */
	public boolean resumeMigration(int currentTick) {
		int duration = Uniform.staticNextIntFromTo(1, 3);
		if (this.lastVisit + duration <= currentTick) {
			this.setOnVisit(false);
			this.household.getCurrentMembers().remove(this);
			/*			resetMyCouples();
			if (this.maritalStatus == Settings.MARITAL_STATUS.MARRIED) {
				Agent spouse = (Agent) model.getAgentMap().get(spouseID);
				if (spouse != null) {
					this.sexualPartners.add(spouse);
					spouse.getSexualPartners().add(this);
				}									
			}
			 */			return true;
		}
		return false;
	}

	//SJA/RM
	//TODO: Finish migration due to mining - returning agents
	/**
	 * Migration stopped; returned to village.
	 */
	public boolean stopMigration(int currentTick)  {
		if (!migrated) {
			return false;
		}
		if (this.age >= 55 
				||
				(this.achievedEducation.compareTo("tertiary") != 0
						//RM 
						&& Math.random() < 0.01
				)) {
			this.migrated = false;
			this.migratedTick = -1;
			this.lastMigrationTry = currentTick;
			this.remittance_Amount = 0;			
			household.getCurrentMembers().add(this);
			return true;
		}				
		else {
			return false;
		}
	}

	public ArrayList<Agent> getOffSprings() {
		return offSprings;
	}
	
	/**If birth possible for the female agent at this time
	 * 
	 * @param currenTick
	 * @return
	 */
	public boolean birthPossible(int currenTick) {
		if (this.gender == Settings.GENDER.FEMALE
				&& isPregnant()) {
			int delta = currenTick - this.pregnancyTick;
			double chance = Math.random();
			if (delta < 7) {
				return false;				
			}
			else if (delta >= 7 
					&& delta < 9
					&& chance < 0.20
			) {
				return true;
			}
			else if (delta >= 9 
					&& delta <= 10
					&& chance < 0.70 
			) {
				return true;
			}
			else {
				//abortion!!
				setPregnant(false);
				model.getPregnantFemales().remove(this);
				return false;
			}
		}
		return false;
	}
	/**Return my spouse if married*/
	public Agent getSpouse() {
		Agent spouse = null;
		if (this.maritalStatus == Settings.MARITAL_STATUS.MARRIED) {			
			spouse = model.getAgentMap().get(new Integer(this.spouseID));			
		}
		return spouse;	
	}
	/**Am I single mother */
	public boolean isSingleMother() {
		return singleMother;
	}
	/**Set single mother */
	public void setSingleMother(boolean singleMother) {
		this.singleMother = singleMother;
	}
	/**is pregnant*/
	public boolean isPregnant() {
		return pregnant;
	}

	public void setPregnant(boolean pregnant) {
		this.pregnant = pregnant;
	}
	/**Get the father id of the child I have delivered */
	public int getFatherOfMyChildID() {
		return fatherOfMyChildID;
	}
	/**Set the father id of the child I have delivered */
	public void setFatherOfMyChildID(int fatherOfMyChildID) {
		this.fatherOfMyChildID = fatherOfMyChildID;
	}

	public int getDeathTick() {
		return deathTick;
	}

	public void setDeathTick(int deathTick) {
		this.deathTick = deathTick;
	}
	/**Initialize age groups  */
	public void initializeAgeGroup() {
		if (getAge() <= Settings.CHILD_AGE_LIMIT) {
			setAgeGroup(Settings.AGE_GROUP.CHILD);
		}
		else if (getAge() > Settings.CHILD_AGE_LIMIT && getAge() <= Settings.ADULT_AGE_LIMIT) {
			setAgeGroup(Settings.AGE_GROUP.ADULT);
		}
		else if (getAge() > Settings.ADULT_AGE_LIMIT) {
			setAgeGroup(Settings.AGE_GROUP.SENIOR);
		}

	}
	/**Remove all my sexual relations */
	public void resetMyCouples() {
		if (this.sexualPartners.isEmpty()) {
			return;
		}
		ArrayList<Couple> temp = new ArrayList<Couple>(model.getPartnersList());

		for (Couple couple : temp) {
			if (this.gender == Settings.GENDER.FEMALE) {
				if (couple.getFemale().getId() == this.id) {
					model.removeCouple(couple);
				}
			}
			else if (this.gender == Settings.GENDER.MALE) {
				if (couple.getMale().getId() == this.id) {
					if (couple.getFemale().isPregnant()
							&& couple.getFemale().getFatherOfMyChildID() == this.id) {
						couple.getFemale().setSingleMother(true);
					}
					model.removeCouple(couple);
				}
			}
		}
	}
	/** Set as pregnant at this step*/
	public void makePregnanct(int currentTick) {
		if (!model.getPregnantFemales().contains(this)) {
			setPregnancyTick(currentTick);
			setPregnant(true);			
			model.getPregnantFemales().add(this);
		}		
	}
	/** Is pregnancy possible*/
	public boolean pregnancyPossible(int currentTick) {
		if (!isPregnant()
				//http://www.prb.org/pdf06/06WorldDataSheet.pdf
				//magic number
				&& Math.random() < 0.60
				&& fertility(currentTick)) {
			return true;
		}
		else {
			return false;	
		}	
	}
	/**
	 * Is having HIV/AIDS?
	 * @return
	 */
	public boolean hasHIV() {
		if (this.healthStatus == Settings.HEALTH_STATUS.EARLY_HIV
				|| this.healthStatus == Settings.HEALTH_STATUS.MIDDLE_HIV
				|| this.healthStatus == Settings.HEALTH_STATUS.LATE_HIV
				|| this.healthStatus == Settings.HEALTH_STATUS.AIDS) {
			return true;
		}
		else {
			return false;
		}
	}
	/**
	 * Return lag for Late HIV Stage 
	 */
	public int getLateLag() {
		return lateLag;
	}
	/**Set lag for Late HIV Stage */
	public void setLateLag(int lateLag) {
		this.lateLag = lateLag;
	}
	/**Get lag for mid HIV Stage */
	public int getMidLag() {
		return midLag;
	}
	/**Set lag for mid HIV Stage */
	public void setMidLag(int midLag) {
		this.midLag = midLag;
	}
	/**Remove this friend from my friendship network*/
	public void removeFriend(Agent friend) {
		if (!model.getGui().getAgentFriendNodeMap().containsKey(new Integer(friend.getId()))) {
			return;
		}
		AgentNode friendNode = (AgentNode) model.getGui().getAgentFriendNodeMap().get(new Integer(
				friend.getId())).node;

		AgentNode myNode = (AgentNode) model.getGui().getAgentFriendNodeMap().get(new Integer(this.id)).node;

		myNode.removeNode(friendNode);
		friendNode.removeNode(myNode);

		if (this.friends.contains(friend)) {
			this.friends.remove(friend);
		}

		if (friend.getFriends().contains(this)) {
			friend.getFriends().remove(this);
		}
	}
	/**Return my female friends */
	public ArrayList<Agent> returnFemaleFriends() {
		ArrayList<Agent> femaleFriends = new ArrayList<Agent>();
		for (Agent friend : getFriends()) {
			if (friend.getGender() == Settings.GENDER.FEMALE
//					&& friend.getAgeGroup() == Settings.AGE_GROUP.ADULT
					&& friend.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
					&& friend.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
					&& !femaleFriends.contains(friend)) {
				femaleFriends.add(friend);
				for (Agent secondaryFriend : friend.getFriends()) {
					if (secondaryFriend.getGender() == Settings.GENDER.FEMALE
//							&& secondaryFriend.getAgeGroup() == Settings.AGE_GROUP.ADULT
							&& secondaryFriend.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
							&& friend.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
							&& !femaleFriends.contains(secondaryFriend)) {
						femaleFriends.add(secondaryFriend);
					}
				}
			}
		}
		for (Agent friend : getFriends()) {
			for (Agent memberHH : friend.getHousehold().getCurrentMembers()) {
				if (	memberHH.getId() != friend.getId()
						&& memberHH.getGender() == Settings.GENDER.FEMALE
						&& friend.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
						&& friend.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
						&& !femaleFriends.contains(friend)) {
					femaleFriends.add(memberHH);
				}
			}
		}
		return femaleFriends;
	}
	/**Return my female acquaintances */
	public ArrayList<Agent> returnFemaleAcquaintances() {
		ArrayList<Agent> femaleAcquaintances = new ArrayList<Agent>();
		for (Agent acquaintance : getAcquaintances()) {
			if (acquaintance.getGender() == Settings.GENDER.FEMALE
					&& acquaintance.getAge() >= Settings.WOMEN_SEX_MINIMUM_AGE
					&& acquaintance.getAge() <= Settings.WOMEN_SEX_MAXIMUM_AGE
					&& !getFriends().contains(acquaintance)) {
				femaleAcquaintances.add(acquaintance);
			}
		}
		return femaleAcquaintances;
	}
	/** Am I currently in village*/
	public boolean availableAtVillage() {
		if (!isMigrated() 					
				|| (isMigrated() && isOnVisit())) {
			return true;
		}
		else {
			return false;
		}
	}
	/**Get my acquaintances list */
	public ArrayList<Agent> getAcquaintances() {
		return acquaintances;
	}
	/**Set my acquaintances list */
	public void setAcquaintances(ArrayList<Agent> acquaintances) {
		this.acquaintances = acquaintances;
	}

	public ArrayList<Message> getSexualPartnershipMessageInbox() {
		return sexualPartnershipMessageInbox;
	}

	public void setSexualPartnershipMessageInbox(ArrayList<Message> sexInbox) {
		this.sexualPartnershipMessageInbox = sexInbox;
	}

	public ArrayList<Message> getSexualPartnershipMessageOutbox() {
		return sexualPartnershipMessageOutbox;
	}

	public void setSexualPartnershipMessageOutbox(ArrayList<Message> sexOutbox) {
		this.sexualPartnershipMessageOutbox = sexOutbox;
	}
	/**Get my potential female partners list */
	public ArrayList<Agent> getPotentialFemales() {
		return potentialFemales;
	}
	/**Set my potential female partners list */
	public void setPotentialFemales(ArrayList<Agent> potentialFemales) {
		this.potentialFemales = potentialFemales;
	}
	/**Get my chance for disability grant */
	public boolean isWouldGetDisabilityGrant() {
		return wouldGetDisabilityGrant;
	}
	/**Set my chance for disability grant */
	public void setWouldGetDisabilityGrant(boolean wouldGetDisabilityGrant) {
		this.wouldGetDisabilityGrant = wouldGetDisabilityGrant;
	}
	/**Get my chance for child grant */
	public boolean isWouldGetChildGrant() {
		return wouldGetChildGrant;
	}
	/**Set my chance for child grant */
	public void setWouldGetChildGrant(boolean wouldGetChildGrant) {
		this.wouldGetChildGrant = wouldGetChildGrant;
	}
	/**Get my chance for pension */
	public boolean isWouldGetPension() {
		return wouldGetPension;
	}
	/**Set my chance for child grant */
	public void setWouldGetPension(boolean wouldGetPension) {
		this.wouldGetPension = wouldGetPension;
	}
}