package units;

import infrastructure.Settings;

public class Couple {
	private Agent male = null, female = null;
	private int id = -1;
	private int lastBirthTick = 0;
	private boolean isMarried = false;
	private int numMonthsDating = 0;
	private boolean isBroken = false;
	private int coupleBeginTick = -1;

	public Agent getFemale() {
		return female;
	}

	public void setFemale(Agent female) {
		this.female = female;
	}

	public Agent getMale() {
		return male;
	}

	public void setMale(Agent male) {
		this.male = male;
	}

	public Couple() {
	}

	public Couple(int _id, Agent _male, Agent _female) {
		this.id = _id;
		this.male = _male;
		this.female = _female;
		if (!male.getSexualPartners().contains(female)) {
			male.getSexualPartners().add(female);
		}
		if (!female.getSexualPartners().contains(male)) {
			female.getSexualPartners().add(male);
		}		
	}
	
	public boolean exists() {
		// of course, this is trivial ...
		if ((male == null) || (female == null) || (!male.isAlive())
				|| (!female.isAlive())) {
			return false;
		}
		return true;
	}
	
	public boolean pregnancyPossible(int currentTick) {
		if (!female.isPregnant()
				&& female.fertility(currentTick)) {
			return true;
		}
		else {
			return false;	
		}		
	}

	public void update() {
		numMonthsDating++;
		if (shouldBreakRelationship()) {
			male.courtshipBroken(female);
			female.courtshipBroken(male);
			this.isBroken = true;
		}
	}
	
	public boolean shouldMarry() {
		if (numMonthsDating >= male.getCourtShipDuration()
				&& numMonthsDating >= female.getCourtShipDuration()
				&& male.getMaritalStatus() != Settings.MARITAL_STATUS.MARRIED
				&& female.getMaritalStatus() != Settings.MARITAL_STATUS.MARRIED) {
//			this.isMarried = true;
			return true;
		}
		else {
			return false;
		}
				
	}
	
	public boolean shouldBreakRelationship() {
		double chance = 0;
		if (male.getSpouseID() == female.getId()
				|| female.getSpouseID() == male.getId()) {
			return false;		
		}	
		
		if (this.male.isOutsider())  {
			return outsiderBreakRelationship();
		}
//		double maleChance = 0.8
//		double maleChance = 0.25 
//		double maleChance = 0.75
		double maleChance = 0.8
//							* ((double)male.getSexualPartners().size()/male.getMaxSexualPartners())
//							* (1 - (double) (male.getAge()/100));
		* ((double)male.getSexualPartners().size()/male.getMaxSexualPartners())
		* (1 - (double) (male.getAge()/male.getExpectedLife()));

		
//		double femaleChance = 0.15
//		double femaleChance = 0.4
//		double femaleChance = 0.3
//		double femaleChance = 0.6
//						* ((double)female.getSexualPartners().size()/female.getMaxSexualPartners())
//						* (1 - (female.getAge()/100));
		double femaleChance = 0.6
		* ((double)female.getSexualPartners().size()/female.getMaxSexualPartners())
		* (1 - (female.getExpectedLife()/100));		

		chance = maleChance * femaleChance;
		
//		System.out.println("------------------------------ ---- FC: " + femaleChance
//				 + " - MC: " + maleChance + " - Net Chance: " + chance);
		return Math.random() < chance ? true : false;
	}
	
	public boolean outsiderBreakRelationship() {
//		magic number
		if (numMonthsDating < 2) {
			return Math.random() < 0.20 ? true : false;
		}
		else if (numMonthsDating >= 2 && numMonthsDating < 5) {
			return Math.random() < 0.35 ? true : false;
		}
		else if (numMonthsDating >= 5 && numMonthsDating < 7) {
			return Math.random() < 0.35 ? true : false;
		}		
		else {
			return Math.random() < 0.10 ? true : false;
		}
	}


	public int getLastBirthTick() {
		return lastBirthTick;
	}

	public void setLastBirthTick(int lastBirthTick) {
		this.lastBirthTick = lastBirthTick;
	}

	public boolean isMarried() {
		return isMarried;
	}

	public void setMarried(boolean married) {
		this.isMarried = married;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getNumMonthsDating() {
		return numMonthsDating;
	}

	public void setNumMonthsDating(int numMonthsDating) {
		this.numMonthsDating = numMonthsDating;
	}

	public boolean isBroken() {
		return isBroken;
	}

	public void setBroken(boolean isBroken) {
		this.isBroken = isBroken;
	}

	public int getCoupleBeginTick() {
		return coupleBeginTick;
	}

	public void setCoupleBeginTick(int coupleBeginTick) {
		this.coupleBeginTick = coupleBeginTick;
	}
}